module( ... or "wildixp", package.seeall)

hints = {
	--[[
	======================
	Wildix phone (WARN: answer is 200 OK by request to /)
	----------------------
		HTTP/1.1 401 Unauthorized.
		Server: mini_httpd/1.19 19dec2003.
		Date: Fri, 01 Nov 2013 09:53:30 GMT.
		Cache-Control: no-cache,no-store.
		WWW-Authenticate: Basic realm="Wildix".
		Content-Type: text/html; charset=%s.
		Connection: close
	]]
	{['server'] = 'mini_httpd.*'}
--	['www-authenticate'] = 'Basic realm="Wildix"',	--TODO: need 401 answer (instead 200)
}

--- Get device settings and username from devdb by mac
local function get_settings (mac)
	local connection = require"common.core".connection
	if connection and mac and mac ~= "" then
		connection:switch("devdb")
		local sql = "SELECT a1.value, a2.value FROM autoprov AS a1 LEFT JOIN autoprov AS a2 ON (a1.mac = a2.mac AND a2.param=%q) WHERE a1.param=%q AND a1.mac=%q"
		local cursor = connection:execute(sql, "settings", "username", mac)
		local username, settings_json = cursor:fetch(); cursor:close()
		return settings_json, username
	end
end

local function convert(keys, ...)
    local h       = require"common.helper"
    local core    = require"common.core"
    local wmsldap = require"core.wmsldap"
	local extenNumber = h.get_extenNumber_str()

	local modeset = select(1, ...) or ""

	local result = {}
	if modeset ~= "html" then
		result = {
			["vpPhone"] = {
				["vpPhone.ini"] = {}
			}
		}
	else
		result = {
			["lineKeys"] = {},
			["stdKeys"] = {},
			["expKeys"] = {}
		}
	end

	local model = select(2, ...)
	local is_new_firmware = select(3, ...) or false
	local is_wp600 = select(4, ...) or false

	local tklimit = {
		wp600 = {
			maxLines    = 6,
			maxStd      = 10,
			maxExp      = 38,
			linesApply  = "0,1,2,3,4,5",
			stdApply    = "0,1,2,3,4,5,6,7,8,9"
		},
		wp500 = {
			maxLines    = 6,
			maxStd      = 10,
			maxExp      = 38,
			linesApply  = "0,1,2,3,4,5",
			stdApply    = "0,1,2,3,4,5,6,7,8,9"
		},
		wp490 = {
			maxLines    = 3,
			maxStd      = 10,
			maxExp      = 38,
			linesApply  = "0,1,2",
			stdApply    = "0,1,2,3,4,5,6,7,8,9"
		},
		wp480 = {
			maxLines    = 3,
			linesApply  = "0,1,2"
		},
		wp400 = {
		}
	}

	if model ~= nil then
		-- lineKeys:
		--   1: LINE
		-- stdKeys:
		--   1: EXT 700
		--   2: SWITCH 2
		-- expKeys:
		--   expMod1:
		--     1: EXT 800
		--     2: CONFERENCE
		--     3: VMACCESS
		--     4: ONTOUCHRECORD
		--     5: ONTOUCHRECORD

		-- Normalization with max line keys on the phone
		for i=1, (tklimit[model]["maxLines"] or 0) do
			if not keys["lineKeys"][i] then
				if keys["stdKeys"] and keys["stdKeys"][1] then
					keys["lineKeys"][i] = keys["stdKeys"][1]
					table.remove(keys["stdKeys"], 1)
				end
			end
		end

		-- lineKeys:
		--   1: LINE
		--   2: EXT 700
		--   3: SWITCH 2
		-- stdKeys:
		-- expKeys:
		--   expMod1:
		--     1: EXT 800
		--     2: CONFERENCE
		--     3: VMACCESS
		--     4: ONTOUCHRECORD
		--     5: ONTOUCHRECORD

		-- Set unused line keys as LINE
		for i=1, (tklimit[model]["maxLines"] or 0) do
			if not keys["lineKeys"][i] then
				keys["lineKeys"][i] = "LINE " .. i
			end
		end

		-- Set unused std keys as LINE
		for i=1, (tklimit[model]["maxStd"] or 0) do
			if not keys["stdKeys"][i] then
				keys["stdKeys"][i] = "LINE"
			end
		end

		-- Reset 2 exp modules
		for i=1, (tklimit[model]["maxExp"] or 0) do
			if not keys["expKeys"]["expMod1"] then
				keys["expKeys"]["expMod1"] = {}
			end

			if not keys["expKeys"]["expMod2"] then
				keys["expKeys"]["expMod2"] = {}
			end

			if not keys["expKeys"]["expMod1"][i] then
				keys["expKeys"]["expMod1"][i] = "LINE"
			end

			if not keys["expKeys"]["expMod2"][i] then
				keys["expKeys"]["expMod2"][i] = "LINE"
			end
		end
	end

	local ext_counter, ext_key_counter = 0, 0

	local events = {
		TRANSFER    = 3,
		CONFERENCE  = 1,
		HOLD        = 4,
		DND         = 5,
		REDIAL      = 6,
		SMS         = 8
	}

	local tdefault = {
		type        = "",
		Value       = "",
		Line        = "0",
		Label       = "",
		KEY_MODE    = "Asterisk",
		Callpickup  = "",
		HotLineId   = "-1",
		PickupNumber = "",
		PickupValue = "",
		HotNumber   = "",
		DKtype      = "0"
	}

	local tskip = {
		"KEY_MODE",
		"PickupNumber",
		"Callpickup",
		"Hotlineid",
		"HotNumber"
	}

	local function specify(key, keytype, label)
		local h, events, tdefault, is_new_firmware, is_wp600 = h, events, tdefault, is_new_firmware, is_wp600
        
		local n = h.split(key, " ")
		local type, value, additional = n[1] or "", n[2] or "", n[3] or ""

		local is_linekey        = keytype == "line"
		local is_expkey         = keytype == "exp"

		-- WEB-302, set empty BLF as EMPTY
		if type == "EXT" and value == "" then
			type = "EMPTY"
		end

		local defDKtype         = (is_linekey and not is_new_firmware) and 13 or 16
		local defValue          = string.format("%s*%s", type:lower(), value)
		local defPickup         = string.format("pickup*%s", value)
		local defQcall          = is_wp600 and 'qcall*' or 'sip:qcall*'
		local defExtValue       = value

		local t = ({
			["SMS"]             = {DKtype = events[type]},
			["REDIAL"]          = {DKtype = events[type]},
			["DND"]             = {DKtype = events[type]},
			["HOLD"]            = {DKtype = events[type]},
			["CONFERENCE"]      = {DKtype = events[type]},
			["TRANSFER"]        = {DKtype = events[type]},

			["EMPTY"]           = {DKtype = is_linekey and 15 or 0},
			["LINE"]            = {DKtype = is_linekey and 15 or 0, Line = defExtValue ~= "" and defExtValue or 0},
			["VMACCESS"]        = {HotNumber = defValue, DKtype = 12},
			["PICKUP"]          = {Value = defPickup, HotNumber = value, DKtype = 13},
			["GPICKUP"]         = {Value = defValue, HotNumber = value, DKtype = 13},
			["SD"]              = {Value = value, HotNumber = value, DKtype = 13},
			["ONTOUCHRECORD"]   = {Value = "rec", DKtype = 11},
			["DTMF"]            = {Value = value, DKtype = 11},
			["LOCALGROUP"]      = {Value = value, DKtype = 45, Label = additional},

			["EXT"]             = {type = "blf", Value = defExtValue, EnablePickup = 1, PickupNumber = value, PickupValue = (is_new_firmware and defExtValue or defPickup), DKtype = defDKtype, Label = additional},
			["TRUNK"]           = {type = "blf", Value = defExtValue, EnablePickup = 1, PickupNumber = value, PickupValue = defPickup, DKtype = defDKtype},

			["STIME"]           = {type = "blf", Value = "stime*" .. value .."*1", EnablePickup = 1, PickupNumber = value, PickupValue = "stime*" .. value .."*1", DKtype = defDKtype},
			["SWITCH"]          = {type = "blf", Value = "switch*" .. value .."*1", EnablePickup = 1, PickupNumber = value, PickupValue = "switch*" .. value .."*1", DKtype = defDKtype},
			["3SWITCH"]         = {type = "blf", Value = "3switch*" .. value .."*1", EnablePickup = 1, PickupNumber = value, PickupValue = "3switch*" .. value .."*1", DKtype = defDKtype},

			["QCALL"]           = {type = "blf", Value = "cg_" .. value, EnablePickup = 1, PickupNumber = value, PickupValue = "pickup*" .. value, DKtype = defDKtype},
			["QPICKUP"]         = {type = "blf", Value = "cg_" .. value, EnablePickup = 1, PickupNumber = value, PickupValue = "pickup*" .. value, DKtype = defDKtype},	-- Not used keys and saved only for compatibility
		})[type] or {type = "blf", Value = defExtValue, DKtype = defDKtype}

		local n = {}
		for k, v in pairs(tdefault) do
			if (is_expkey and not h.in_table(k, tskip)) or not is_expkey then
				n[k] = t[k] or tostring(v)
				if k == 'Label' and label ~= nil and n["Label"] == nil then
					n["Label"] = label
				end
			end
		end

		return n
	end

	local function encode_key (key, keytype, is_new_firmware, label)
		local result

		-- Any key
		if keytype ~= "line" then
			result = table.concat ({
				(key["DKtype"] ~= 16 and not is_new_firmware) and 1 or key["DKtype"],
				key["DKtype"] ~= 12 and 0 or 1,
				is_new_firmware and label or (key["DKtype"] ~= 16 and key["DKtype"] or 1),
				key["Value"],
				key["PickupValue"],
				""
			}, "$")
			-- LINE key
		else
			result = table.concat ({
				(key["DKtype"] ~= 16 or is_new_firmware) and key["DKtype"] or 13,
				key["DKtype"] ~= 12 and (key["Line"] ~= 6 and 0 or 6) or 1,
				is_new_firmware and label or (key["DKtype"] ~= 16 and key["DKtype"] or 1),
				key["Value"],
				key["PickupValue"],
				""
			},"$")
		end

		return result
	end

	-- Line keys
	if is_new_firmware and #keys["lineKeys"] > 0 and result["lineKeys"] and tklimit[model]["linesApply"] then
		table.insert(result["lineKeys"], tklimit[model]["linesApply"])
	end
	for i, v in ipairs(keys["lineKeys"]) do
		if i > 6 then break end

		local exten = v:match("(%d+)")
		local label = ""
		if exten then
			local person, tmp = {}
			local res = wmsldap:get(extenNumber, tostring(exten))
			if #res ~= 0 then
				tmp, person = unpack(res[1])
				label = person["cn"]
			else
				label = v
			end
		end

		local key = specify(v, "line", label)

		if modeset ~= "html" then
			result["vpPhone"]["vpPhone.ini"]["memory" .. (i+10)] = key
		else
			table.insert(result["lineKeys"], encode_key(key, "line", is_new_firmware, label))
		end
	end

	-- standart keys
	if is_new_firmware and #keys["stdKeys"] > 0 and result["stdKeys"] and tklimit[model]["stdApply"] then
		--table.insert(result["stdKeys"], tklimit[model]["stdApply"]:sub(0, tklimit[model]["stdApply"]:find(#keys["stdKeys"])-2))
		table.insert(result["stdKeys"], tklimit[model]["stdApply"])
	end
	for i, v in ipairs (keys["stdKeys"]) do
		if i > 10 then break end

		local key = specify (v, "std", nil)

		if modeset ~= "html" then
			result["vpPhone"]["vpPhone.ini"]["memory" .. i] = key
		else
			table.insert(result["stdKeys"], encode_key (key, "std", is_new_firmware, ""))
		end
	end

	-- Exp modules
	for expname, tkeys in pairs (keys["expKeys"]) do
		local serial = "0000000000000" .. expname:match("expMod(%d+)")
		local extstring = "Ext38_" .. serial .. ".cfg"

		if modeset == "html" and not result["expKeys"][serial] then
			result["expKeys"][serial] = {}
		end

		if is_new_firmware and modeset == "html" then
			table.insert(result["expKeys"][serial], "0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35,36,37")
		end

		for i, v in ipairs (tkeys) do
			if i > 38 then break end

			local key = specify (v, "exp", nil)

			if modeset ~= "html" then
				if not result["vpPhone"][extstring] then
					result["vpPhone"][extstring] = {}
				end
				result["vpPhone"][extstring]["Key" .. (i-1)] = key -- start with "0"
			else
				table.insert(result["expKeys"][serial], encode_key (key, "exp", is_new_firmware, ""))
			end
		end
	end
	return result
end

function verify (host, port)
	local logger     = logger
	local http       = require"core.httpClient":new()
	local xml        = require"core.htmlparser"
	local helper     = require"common.helper"
	local connection = require"common.core".connection
	local pbxutils   = require"core.pbxutils"

	local gpassword        = helper.get_gpassword()
	local is_ssl_negotiate = 443 == port.number
	local PORT_NUMBER      = is_ssl_negotiate and not gconf.HTTPForceUse and 443 or 80
	local t                = {["ssl"] = is_ssl_negotiate}
	local auth_table       = {"admin:admin", "admin:" .. gpassword}

	local sql, cursor, token_pass, dublicate_ip
	if host.mac_addr_str == "" then
		sql = [[ SELECT token, count(id) FROM autoprov WHERE param=%q AND value LIKE '%%"address":%%"%s"%%' AND value LIKE '%%"model":%%"wp%%"%%' ]]
		cursor = connection:execute(sql, "device", host.ip or "")
		token_pass, dublicate_ip = cursor:fetch(); cursor:close()
		if tonumber(dublicate_ip) > 1 then
			logger:error("Found %s duplicated address %q at provisioning database", dublicate_ip, host.ip or "")
			return t
		end
	else
   		sql = "SELECT token FROM autoprov WHERE param=%q AND mac=%q"
    	cursor = connection:execute(sql, "device", host.mac_addr_str)
    	token_pass = cursor:fetch(); cursor:close()
    end
    
    t["token_type"] = helper.is_new_token(token_pass)
    if t["token_type"] == "new" then
    	table.insert(auth_table, 1, "admin:"..token_pass)
    	t["token"] = token_pass
    end
    logger:debug(auth_table, "auth_table")
    
    -- Checking logo link
    local answer = http:rawget(host.ip, PORT_NUMBER, "/js/configfile.js")
    if not answer or 404 == answer.code then
        answer = http:rawget(host.ip, PORT_NUMBER, "/js/common.js")
    end
    if answer and 404 ~= answer.code then
        local logo_link = answer.body:match('.*var logo_link="([^%"]*)".*')
        if not logo_link:find("wildix") then
            logger:debug("Can't detect vendor by request to logo link")
            return false
        end
    else
        logger:error("Host '%s' does not answer to logo request", host.ip)
        return t
    end
    
    -- Get device status information
    local is_wp600 = nil  -- Check WP600
    for _, authpair in pairs(auth_table) do
        is_wp600          = false
        local login, pass = authpair:match("(.*):(.*)")
        answer = http:rawget(host.ip, PORT_NUMBER, "/cgi-bin/ConfigManApp.com?Id=1", { authUser = login, authPass = pass })
        if 200 == answer.code and not answer.body:find("not authorized to access") then
            t["auth"] = {user = login, pass = pass}
            break
        end
        answer = http:rawget(host.ip, PORT_NUMBER, "/cgi-bin/cgiServer.exx?page=Status.htm", { authUser = login, authPass = pass })
        if 200 == answer.code and not answer.body:find("not authorized to access") then
            is_wp600 = true
            t["auth"] = {user = login, pass = pass}
            break
        end
    end
    if not t["auth"] then
        logger:error("Wrong authorization for host '%s'", host.ip)
        return t
    end

	local firmware, model -- model detects as for Yealink
	local doc = xml:new(answer.body)
	if is_wp600 then
		local s   = doc:find('//script[contains(text(),"var strIfconfig")]')
		local mac = pbxutils.split(s, "HWaddr ")[2]:sub(1, 17)
		if host.mac_addr_str == "" then
			host.mac_addr_str = mac:gsub("([0-9A-Fa-f]+)([\:]*)", "%1"):lower()
		end
		s        = doc:find('//script[contains(text(),"var objWanType")]')
		firmware = pbxutils.split(s, "innerHTML = ")[2]:match('(38.%d+.%d+.%d+)')
		model    = "t38"
	else
		local s = doc:find('//script[contains(text(),"var Cfgdata")]'):match('Cfgdata="(.+)"')
		local data = pbxutils.split(s, "Ã¾")
		if #data == 1 then
			data = pbxutils.split(s, "þ")
		end
		firmware = data[9]
		if host.mac_addr_str == "" then
			host.mac_addr_str = data[5]:gsub("([0-9A-Fa-f]+)([\-]*)", "%1"):lower()
		end
		model = doc:find('//script[contains(text(),"var devtype")]'):match('devtype="(.-)"'):lower()
		if model == "t22" then	-- there is a mistake in html content, let's check firmware version
			if string.find(data[6], "^18\.") then
				model, firmware = "t18", data[6]
			end
		end
	end
	t["fwversion"] = firmware
	t["mac_addr_str"] = host.mac_addr_str:lower()
	t["model"] = ({
		["t18"] = "wp400",
		["t22"] = "wp480",
		["t26"] = "wp490",
		["t28"] = "wp500",
		["t38"] = "wp600"
	})[model] or ""
	
	return t
end

function provision (host, port, ...)
	local args       = select(1, ...)
	local http       = require"core.httpClient":new()
	local h          = require"common.helper"
	local connection = require"common.core".connection
	
	-- processing registrar address
	local registrar       = nil
	local case            = "hosts"
    local cfg             = {
                FailoverByDns    = gconf.FailoverByDns
               ,ForceResolveName = gconf.ForceResolveName
               ,ForceResolveIp   = gconf.ForceResolveIp
          }
	local addr_t, err_msg = h.resolve_by_hosts(host.ip_src, nil, cfg)
	if not addr_t then
		if err_msg then	-- printing resolving error
			logger:warning("Hosts file was not processed: %s", err_msg)
		end
	else
		registrar = addr_t.domain
	end
	if not registrar then
		registrar, case, err_msg = h.get_registrar(host.ip_src, host.ip) 
	end
	logger:debug("Registrar address %q got from %s%s", registrar, case, err_msg and ". Errors: "..err_msg or "")
	
	local PORT_NUMBER   = host.ssl and not gconf.HTTPForceUse and 443 or 80
	local http_protocol = (gconf.HTTPForceUse or host.model == "wp400") and "http" or "https"
	local http_port     = (gconf.HTTPForceUse or host.model == "wp400") and "80" or "443"
	
	-- Get security password
	local gpassword
	if host.token_type == "new" then
		gpassword = host.token or h.get_gpassword()
	else
		gpassword = h.get_gpassword()
		if host.token_type == "old" then
			logger:info("Token has old format")
		end
	end

	-- Provisioning link
	local provlink
	if host.model ~= "wp600" then
		provlink = string.gsub(
			"$protocol://$registrar:$port/?action=setaps", "%$(%w+)",
			{
				registrar = registrar,
				protocol = http_protocol,
				port = http_port,
			}
		)
	else
		provlink = string.gsub(
			"$protocol://$registrar:$port/?action=setaps", "%$(%w+)",
			{
				registrar = registrar,
				protocol = http_protocol,
				port = http_port,
			}
		)
	end

	local options = {
	     authUser = host.auth.user
	    ,authPass = host.auth.pass
		,timeout  = 2000
	}

	local answer, config_data, data
	if host.model ~= "wp600" then
		-- Get syslog configuration from web interface
        answer = http:rawget(host.ip, PORT_NUMBER, "/cgi-bin/ConfigManApp.com?Id=16", options) or {}
        logger:debug("Getting current syslog settings for %s - %s", host.model, answer.code or 408)
        local syslog_server = string.match(answer.body or "", 'name=ConfigVal6 type=hidden value=\"(.-)\"') or ""
		local syslog_level  = "6"

		-- Set parameters, we must save 128 dhcp option before provisioning
		config_data = {"128", "1", provlink, "wmsdevice", gpassword, syslog_server, "", "1", "", "00:00", "00:00", "", "", "0", "1", "", syslog_level}
		data = "PAGEID=16&CONFIG_DATA=" .. http.urlEncode(table.concat(config_data, "þ"))
		answer = http:rawpost(host.ip, PORT_NUMBER, "/cgi-bin/ConfigManApp.com", data, options) or {}
		logger:debug("Setting autoprovisioning parameters for %s - %s", host.model, answer.code or 408)
		
		-- Run autoprovisioning
		config_data = {"AUTOPROVISION NOW", "128", "1", provlink, "wmsdevice", gpassword, ""}
		data = "PAGEID=16&CONFIG_DATA=" .. http.urlEncode(table.concat(config_data, "þ"))
		answer = http:rawpost(host.ip, PORT_NUMBER, "/cgi-bin/ConfigManApp.com", data, options) or {}
		logger:info("Sending http request to %s for autoprovision now - %s", host.model, answer.code or 408)
	else
		-- Set parameters
		config_data = {
			http.urlEncode('profileOpen("/phone/config/system.ini")'),
			http.urlEncode('profileSetString("AutoProvision","bEnablePNP","0")'),
			http.urlEncode('profileSetString("AutoProvision","bEnableDHCPOption","0")'),
			http.urlEncode('profileSetString("AutoProvision","listUserOptions","128")'),
			http.urlEncode('profileSetString("AutoProvision","nUserOptionsType","1")'),
			http.urlEncode('profileSetString("AutoProvision","strServerURL",' .. provlink .. ')'),
			http.urlEncode('profileSetString("AutoProvision","strUser","wmsdevice")'),
			http.urlEncode('profileSetString("AutoProvision","strPassword",' .. gpassword .. ')'),
			http.urlEncode('profileSetString("AutoProvision","bEnablePowerOn","1")'),
			http.urlEncode('profileSetString("AutoProvision","bEnableRepeat","0")'),
			http.urlEncode('profileSetString("AutoProvision","bEnableWeekly","0")'),
			http.urlEncode('profileSave()')
		}
		data = "postCommand=" .. table.concat(config_data)
		answer = http:rawpost(host.ip, PORT_NUMBER, "/cgi-bin/cgiServer.exx", data, options) or {}
		logger:debug("Setting autoprovisioning parameters for %s - %s", host.model, answer.code or 408)

		-- Run autoprovisioning
		data = "postCommand=" .. http.urlEncode('msgSendMessage("autoServer","0xA0000","0","0")')
		answer = http:rawpost(host.ip, PORT_NUMBER, "/cgi-bin/cgiServer.exx", data, options) or {}
		logger:info("Sending http request to %s for autoprovision now - %s", host.model, answer.code or 408)
	end
end

function configuration (agent, ...)
	local json       = require"dkjson"
	local core       = require"common.core"
	local h          = require"common.helper"
	local wmsldap    = require"core.wmsldap"
	local pbxutils   = require"core.pbxutils"
	local wmsperm    = require"common.permissions"
	local http       = require"core.httpClient"
	local connection = core.connection
	local logger     = logger
	local err
	
	
	-- getting registrar address
	local registrar       = agent["registrar"]
	local registrar_ip 	  = false
    local cfg             = {
                FailoverByDns    = gconf.FailoverByDns
               ,ForceResolveName = gconf.ForceResolveName
               ,ForceResolveIp   = gconf.ForceResolveIp
          }
	local addr_t, err_msg = h.resolve_by_hosts(registrar, nil, cfg)
	if not addr_t then
		if err_msg then
			logger:warning("Hosts file was not processed: %s", err_msg)
		end
	else
		logger:debug(addr_t, "hosts_registrar")
		if addr_t.domain then
			registrar = addr_t.domain
		end
		if addr_t.ip then
			registrar_ip = addr_t.ip
		end
	end

	local is_wp600 = agent["model"] == "wp600" and true or false
	local resource = is_wp600 and core.load_resource("wp600") or core.load_resource("wildixp")
	local result   = pbxutils.tcopy(resource.parameters) or {}

    -- retrieve device information
    local token_pass = core.proceedDeviceInfo(agent).token
	
	local settings_json, username = get_settings(agent["mac"])
	if not username then
		username = core.register(agent["mac"])
		if not username then
			do return "", "", "PROVISIONING IS BLOCKED" end
		end
	end

	local extenNumber = h.get_extenNumber_str()
	local res = wmsldap:get(extenNumber, username)
	local person = {}

	if not res then
		-- Somthing with LDAP?
		return {}, username, "Could not connect to LDAP"
	elseif #res == 0 and not username:find("unknown%d") then
		-- No user in LDAP?
		return {}, username, string.format("Could not get information for %q from LDAP", username)
	elseif #res ~= 0 then
		_, person = unpack(res[1])
	end

	local secret = person["wuserPasswordSip"] or "1234"
	if username:find("unknown%d") then
		local rv, err = h.pass_for_unknown(username)
		if err then
			logger:error("password generation error for unknown account:", err)
		end
		secret = rv
	end
	if person["preferredLanguage"] == nil then
		person["preferredLanguage"] = h.get_admin_lang()
	end

	local uid = person.uid or ""
	local user_features = require"common.userFeatures.userFeatures":load(uid)
	local keys = core.fkeys_get(user_features)

	local is_addr_remote = not h.isLocalIP(agent["address"])
	local sip_port       = "5060"
	local sip_transport	 = "UDP"
	local http_protocol  = agent["ssl"] == "on" and not gconf.HTTPForceUse and "https" or "http"
	local http_port = gconf.HTTPForceUse and "80" or h.get_alternate_port() or agent["registrar:port"]
	if agent["model"] == "wp400" and "https" == http_protocol then
		http_protocol = "http"
		http_port     = "80"
	end
	
	local is_wildix_firmware = false
	local is_new_firmware    = false
	local is_newV61_firmware = false
	local is_features_button = false
	local is_codec_priority  = false
	
	local available_major_version      = 0
	local available_minor_version 	   = 0
	local fwResult, err  = h.getRemoteFirmwaresInfo("WP", agent["model"])
	local remoteFwPack = fwResult and fwResult.data
	if remoteFwPack then
		local remoteFwInfo = h.parse_firmware(remoteFwPack["version"])
		if remoteFwInfo then
			available_major_version = remoteFwInfo["major"]
			available_minor_version = remoteFwInfo["minor"]
		end
	elseif err then
		logger:warning("%s %s", err, fwResult.tag)
	end

	-- Version for not wp600 wildix phones
	local is_wildix_firmware = false
	local is_new_firmware	 = false
	local is_newV61_firmware = false
	local is_features_button = false
	-- Version for wp600 
	local is_v14_wp600_fmw 	 = false
	local is_v21_wp600_fmw 	 = false
	
	local model_spec		 = nil
	local major_version		 = 0
	local minor_version		 = 0
	local devFwInfo			 = h.parse_firmware(agent["fw:version"])
	if devFwInfo then
		model_spec 	  = devFwInfo["model"]
		major_version = devFwInfo["major"]
		minor_version = devFwInfo["minor"]
	end
	if not is_wp600 then	-- check version for not wp600
		if model_spec and major_version >= 51 then
			is_wildix_firmware = true
		end
		if model_spec and major_version >= 60 or available_major_version >= 60 then
			is_new_firmware = true
		end
		if model_spec and major_version >= 61 or available_major_version >= 61 then
			is_newV61_firmware = true
		end
		if agent["model"] ~= "wp400" and ((model_spec and major_version >= 61 and minor_version >= 13) or 
		  (available_major_version >= 61 and available_minor_version)) then
			is_codec_priority = true
		end
	else	-- check version for wp600
		if model_spec and minor_version >= 14 or available_minor_version >= 14 then
			is_v14_wp600_fmw = true
		end
		if model_spec and minor_version >= 17 or available_minor_version >= 17 then
			is_codec_priority = true
		end
		if model_spec and minor_version >= 21 or available_minor_version >= 21 then
			is_v21_wp600_fmw = true
		end
	end
	
	-- set up firmware upload section
	local isUpgradeRunning = false
	if remoteFwPack then
		if agent["fw:version"] ~= remoteFwPack.version then
			logger:info("Firmware (%s) upgrade to %s %s", agent["fw:version"] or "", remoteFwPack["version"], fwResult.tag)
			-- IMPORTANT: we strongly need /yealink/config/Setting/swprotect.cfg for firmware upgrade
			if is_wp600 then
				result["__nopath__"][" cfg:/config/system.ini,reboot=1"]["sw_ver.sw_protect"] = "0"
			else
				result["Setting"]["update.cfg"]["sw_ver"]["sw_protect"] 					  = "0"
			end

			-- Firmware url data
			local fwPath  = string.format("%s/%s.rom", remoteFwPack.path, remoteFwPack.version)
			local initUrl = string.format("%s://%s:%s", http_protocol, registrar, http_port) 		
			
			if is_wp600 then
				result["__nopath__"][" rom:Firmware"]["url"] = initUrl.."/"..fwPath
			else
				result["__nopath__"]["firmware"] = {
					 server_type 	= http_protocol
					,http_url		= initUrl 
					,firmware_name 	= fwPath
				} 
			end
			
			-- update flag about device upgrade in case device located behind NAT
			local cursor 	  = connection:execute("SELECT value FROM autoprov WHERE param='upgrade' AND mac='%s'", agent["mac"])
			local upgradeFlag = cursor:fetch(); cursor:close()
			isUpgradeRunning  = true
			if "remupgrade" == upgradeFlag then
				sql = "INSERT OR REPLACE INTO autoprov(id,param,value,mac) VALUES((SELECT id FROM autoprov WHERE param='upgrade' AND mac='%s'),'upgrade','%s','%s')"
				connection:execute(sql, agent["mac"], os.time(), agent["mac"])
			end
		end
	end
	if not isUpgradeRunning then	-- remove upgrade firmware flag
		connection:execute("DELETE FROM autoprov WHERE param='upgrade' AND mac='%s'", agent["mac"])
	end
	
	if not username:find("unknown%d") then
		if (agent["model"] == "wp500" and (minor_version >= 13 or is_newV61_firmware)) or ((agent["model"] == "wp480" or agent["model"] == "wp490") and (minor_version >= 10 or is_newV61_firmware)) or is_wp600 then
			is_features_button = true
		end
	end

	local keys = convert(keys, "", agent["model"], (is_new_firmware or is_wp600), is_wp600)["vpPhone"]
	if not is_wp600 then
		result["vpPhone"] = keys
	else
		for keysk, keysv in pairs(keys) do
			if keysk == "vpPhone.ini" then
				do
					local tcfg = result["__nopath__"][" cfg:/phone/config/vpPhone/vpPhone.ini,reboot=1"]
					for memk, memv in pairs(keysv) do
						for memek, memev in pairs(memv) do
							tcfg[memk .. "." .. memek] = memev
						end
					end
				end
			else
				local keyska = h.split(keysk, "_")
				result["__nopath__"][" cfg:/phone/config/vpPhone/" .. keyska[1] .. "_" .. keyska[2] .. ",reboot=1"] = {}
				for expk, expv in pairs(keysv) do
					for expek, expev in pairs(expv) do
						result["__nopath__"][" cfg:/phone/config/vpPhone/" .. keyska[1] .. "_" .. keyska[2] .. ",reboot=1"][expk .. "." .. expek] = expev
					end
				end
			end
		end
	end

	local tr_Phonebook = require"xmlbrowser".label_name("phonebook", person["preferredLanguage"])
	if is_new_firmware then
		for i=1, 14 do
			result["vpPhone"]["vpPhone.ini"]["programablekey" .. i] = {}
			result["vpPhone"]["vpPhone.ini"]["programablekey" .. i]["DKtype"] = "0"
		end
		result["vpPhone"]["vpPhone.ini"]["programablekey1"]["DKtype"] = "28"
		result["vpPhone"]["vpPhone.ini"]["programablekey2"]["DKtype"] = "47"
		result["vpPhone"]["vpPhone.ini"]["programablekey2"]["Label"]  = tr_Phonebook
		result["vpPhone"]["vpPhone.ini"]["programablekey4"]["DKtype"] = "30"
		result["vpPhone"]["vpPhone.ini"]["programablekey9"]["DKtype"] = "33"
	end

	--Disable phone menu button by ACL
	if wmsperm:check(person["owner"] or "", "canuse", "PHONE_SETTING_MENU", nil, person["licenseType"]) == "nouse" then
		if is_wp600 then
			result["__nopath__"][" cfg:/phone/config/vpPhone/vpPhone.ini,reboot=1"]["programablekey4.DKtype"] = "0"
			result["__nopath__"][" cfg:/phone/config/vpPhone/vpPhone.ini,reboot=1"]["programablekey4.Label"] = ""
		elseif result["vpPhone"]["vpPhone.ini"]["programablekey4"] then
			result["vpPhone"]["vpPhone.ini"]["programablekey4"]["DKtype"] = "0"
			result["vpPhone"]["vpPhone.ini"]["programablekey4"]["Label"] = ""
		end
	end

	-- XML feature browser
	if is_features_button then
		local link = string.gsub(
			"$protocol://$registrar:$port/?action=getxmlbrowser&get=features", "%$(%w+)",
			{
				protocol  = http_protocol,
				registrar = registrar,
				port      = http_port
			}
		)
		local tr_Features = require"xmlbrowser".label_name("features", person["preferredLanguage"])
		if not is_wp600 then
			result["vpPhone"]["vpPhone.ini"]["programablekey2"]["DKtype"] = "47"
			result["vpPhone"]["vpPhone.ini"]["programablekey2"]["Label"] = tr_Phonebook

			result["vpPhone"]["vpPhone.ini"]["programablekey3"]["DKtype"] = "27"
			result["vpPhone"]["vpPhone.ini"]["programablekey3"]["Value"] = link
			result["vpPhone"]["vpPhone.ini"]["programablekey3"]["Label"] = tr_Features
		else
			result["__nopath__"][" cfg:/phone/config/vpPhone/vpPhone.ini,reboot=1"]["programablekey2.DKtype"] = "47"
			result["__nopath__"][" cfg:/phone/config/vpPhone/vpPhone.ini,reboot=1"]["programablekey2.Label"] = tr_Phonebook

			result["__nopath__"][" cfg:/phone/config/vpPhone/vpPhone.ini,reboot=1"]["programablekey3.DKtype"] = "27"
			result["__nopath__"][" cfg:/phone/config/vpPhone/vpPhone.ini,reboot=1"]["programablekey3.Value"] = link
			result["__nopath__"][" cfg:/phone/config/vpPhone/vpPhone.ini,reboot=1"]["programablekey3.Label"] = tr_Features
		end
	end
	
	-- Phonebooks
	do
		local tcfg = (is_wp600 and result["__nopath__"][" cfg:/phone/config/user.ini,reboot=0"] or result["Setting"]["Setting.cfg"])
		if not is_wp600 then
			for i = 0, 5 do
				if not tcfg["RemotePhoneBook".. i] then
					tcfg["RemotePhoneBook".. i] = {}
				end
				tcfg["RemotePhoneBook".. i]["Name"] = "%NULL%"
				tcfg["RemotePhoneBook".. i]["URL"] = "%NULL%"
			end
		else
			for i = 0, 5 do
				tcfg["RemotePhoneBook".. i ..".Name"] = ""
				tcfg["RemotePhoneBook".. i ..".URL"] = ""
			end
		end
		
		local pbooknames = "searchall,Directory," .. table.concat(core.get_list_phonebook(uid), ",")
		if pbooknames ~= "" then
			local pbookst = h.split(pbooknames, ",")
			local pbookst_names = core.get_phonebook_names(pbookst)
			local link, key, pbookname
			for i, name in ipairs(h.split (pbooknames, ",")) do
				if i == 1 then
					pbookname = "searchall"
				elseif i == 2 then
					pbookname = "Directory"
				else
					pbookname = pbookst_names[name] or ""
				end
				link = string.gsub(
					"$protocol://$registrar:$port/?action=getphonebook&phonebook=$phonebook&input=#SEARCH", "%$(%w+)",
					{
						protocol  = http_protocol,
						registrar = registrar,
						port      = http_port,
						phonebook = http.urlEncode(name:lower())
					}
				)

				if pbookname == "Directory" then
                    pbookname = require"xmlbrowser".label_name("pbdirectory", person["preferredLanguage"])
                elseif "searchall" == pbookname then
                    pbookname = require"xmlbrowser".label_name("pbsearchall", person["preferredLanguage"])
                end
				if i <= 5 then
					key = "RemotePhoneBook" .. (i - 1)
					if not is_wp600 then
						tcfg[key]["Name"] = pbookname
						tcfg[key]["URL"] = link
					else
						tcfg[key .. ".Name"] = pbookname
						tcfg[key .. ".URL"] = link
					end
				end
			end
		end
	end

	do 	-- Network settings
		local t = {}
		local update_db_settings = false
		
		if settings_json then
			t = json.decode(settings_json)
		end

		-- SYSLOG server (by default syslog server is empty)
		if t["syslog_server"] and t["syslog_server"] ~= "" then
			if is_wp600 then
				result["__nopath__"][" cfg:/config/system.ini,reboot=1"]["SYSLOG.SyslogdIP"] = t["syslog_server"]
			else
				result["Network"]["Network.cfg"]["SYSLOG"]["SyslogdIP"] = t["syslog_server"]
			end
		end
		
		-- SNMP access server
		if t["snmp_access"] and t["snmp_access"] ~= "" then
			--- XXX: WP600 have not support SNMP  
			if is_wp600 then
				result["__nopath__"][" cfg:/config/system.ini,reboot=1"]["snmp.snmp_enable"] = "1"
				result["__nopath__"][" cfg:/config/system.ini,reboot=1"]["snmp.snmp_port"] = "161"
				result["__nopath__"][" cfg:/config/system.ini,reboot=1"]["snmp.snmp_trusted_address"] = t["snmp_access"]
			else
				result["Network"]["Network.cfg"]["snmp"]["snmp_enable"] = "1"
				result["Network"]["Network.cfg"]["snmp"]["snmp_port"] = "161"
				result["Network"]["Network.cfg"]["snmp"]["snmp_trusted_address"] = t["snmp_access"]
			end
		end
	
		-- VLAN settings
		if tonumber(t["vid_voice"]) and tonumber(t["vid_voice"]) > 0 then
			if not is_wp600 then
				do local tcfg = result["Network"]["Network.cfg"]["VLAN"]
					tcfg["ISVLAN"]                    = 1
					tcfg["VID"]                       = t["vid_voice"] or "0"
					tcfg["USRPRIORITY"]               = t["qos_voice"] or "7"
					tcfg["CFI"]                       = 0
				end
			else
				do local tcfg = result["__nopath__"][" cfg:/config/system.ini,reboot=1"]
					tcfg["VLAN.ISVLAN"]               = 1
					tcfg["VLAN.VID"]                  = t["vid_voice"] or "0"
					tcfg["VLAN.USRPRIORITY"]          = t["qos_voice"] or "7"
					tcfg["VLAN.CFI"]				  = 0
				end
			end
		end
		if tonumber(t["vid_data"]) and tonumber(t["vid_data"]) > 0 then
			if not is_wp600 then
				do local tcfg = result["Network"]["Network.cfg"]["VLAN"]
					tcfg["PC_PORT_VLAN_ENABLE"]       = 1
					tcfg["PC_PORT_VID"]               = t["vid_data"] or "0"
					tcfg["PC_PORT_PRIORITY"]          = t["qos_data"] or "0"
					tcfg["CFI"]                       = 0
				end
			else
				do local tcfg = result["__nopath__"][" cfg:/config/system.ini,reboot=1"]
					tcfg["VLAN.PC_PORT_VLAN_ENABLE"]  = 1
					tcfg["VLAN.PC_PORT_VID"]          = t["vid_data"] or "0"
					tcfg["VLAN.PC_PORT_PRIORITY"]     = t["qos_data"] or "0"
					tcfg["VLAN.CFI"]				  = 0
				end
			end
		end
		
		-- set default DNS and NTP settings if not define
		t["dns_server"], t["second_dns_server"] = h.get_prim_sec_dns_servers(t["dns_server"], t["second_dns_server"])
		t["ntp_server"] 						= t["ntp_server"] or ""
		
		-- use static ip settings
		local is_addr_present = h.is_addr_present(agent["address"], agent["mac"])	-- check dublicate address
		if t["use_static_ip"] and (is_addr_present or is_addr_remote) then
			if is_addr_present then
				logger:warning("Disable static ip. Other device have the same ip address: %s", agent["address"] or "")
			end
			if is_addr_remote then	-- device behind NAT can't user static ip
				logger:warning("Disable static ip. Device ip address (%s) is placed behind NAT", agent["address"] or "")
			end
			t["use_static_ip"] = nil
			update_db_settings = true
		end
		
		-- get settings for leased device by pbx dhcp
		local is_leased
		if not t["use_static_ip"] or t["dns_server"] == "" or t["ntp_server"] == "" then
			is_leased = h.is_addr_leased(agent["address"])
		end
		if not t["use_static_ip"] and not is_addr_present and is_leased then
			h.set_addr_static(agent["address"], agent["mac"])	-- mark this lease as static
			t["use_static_ip"] = 1
			update_db_settings = true
		end
		if is_leased then
			if "" == t["dns_server"] or "" == t["second_dns_server"] then
				local dns_dhcp1, dns_dhcp2 = h.get_dns_servers()
        		if dns_dhcp1 and "" ~= dns_dhcp1 and "" == t["dns_server"] then
        			t["dns_server"] = dns_dhcp1             -- set primary DNS from DHCP config
        			update_db_settings = true
        		end
        		if dns_dhcp2 and "" ~= dns_dhcp2 and "" == t["second_dns_server"] then
        			t["second_dns_server"] = dns_dhcp2      -- set secondary DNS from DHCP config
        			update_db_settings = true
        		end
			end
			

		    if t["ntp_server"] == "" then -- set NTP from DHCP config
		        local ntp_dhcp = h.get_ntp_servers()
		        if ntp_dhcp and ntp_dhcp ~= "" then
		            t["ntp_server"] = ntp_dhcp
		            update_db_settings = true
		        end
		    end
		end

		if t["use_static_ip"] == 1 then	-- use static ip
			local _, netmask, gateway = h.host_settings(agent["address"], registrar_ip)
			if netmask and gateway then
				if not is_wp600 then
					do local tcfg = result["Network"]["Network.cfg"]["WAN"]
						tcfg["WANType"]               = 2
						tcfg["WANStaticIP"]           = agent["address"]
						tcfg["WANSubnetMask"]         = netmask
						tcfg["WANDefaultGateway"]     = gateway
					end
				else
					do local tcfg = result["__nopath__"][" cfg:/config/system.ini,reboot=1"]
						tcfg["Network.eWANType"]      = 2
						tcfg["Network.strWANIP"]      = agent["address"]
						tcfg["Network.strWANMask"]    = netmask
						tcfg["Network.strWanGateway"] = gateway
					end
				end
				
				if registrar_ip then
    				if "" == t["dns_server"] then
    					t["dns_server"]    = registrar_ip
    					update_db_settings = true
    				end
    				if "" == t["second_dns_server"] then
    					t["second_dns_server"] = registrar_ip
    					update_db_settings     = true
    				end
    			end
    		
				if t["ntp_server"] == "" then
					t["ntp_server"]    = registrar
					update_db_settings = true
				end
			else
				logger:warning("For current network configurations is not possible set static ip, disable it")
				t["use_static_ip"] = nil
				update_db_settings = true
			end
		end
		
		-- update devdb with new settings
		if update_db_settings then
			connection:switch ("devdb")
			local sql = "INSERT OR REPLACE INTO autoprov (id, param, value, mac) VALUES ((SELECT id FROM autoprov WHERE param=%q AND mac=%q), %q, '%s', %q)"
			connection:execute (sql, "settings", agent["mac"], "settings", json.encode(t), agent["mac"])
		end

		-- DNS settings
		if "" ~= t["dns_server"] then
			if not is_wp600 then
				do 
					local tcfg = result["Network"]["Network.cfg"]["DNS"]
					tcfg["PrimaryDNS"] = t["dns_server"]
					if "" ~= t["second_dns_server"] then
						tcfg["SecondaryDNS"] = t["second_dns_server"]
					end
				end
			else
				do 
					local tcfg = result["__nopath__"][" cfg:/config/system.ini,reboot=1"]
					tcfg["Network.strWanPrimaryDNS"] = t["dns_server"]
					if "" ~= t["second_dns_server"] then
						tcfg["Network.strWanSecondaryDNS"] = t["second_dns_server"]
					end
				end
			end
		end

		-- NTP settings 
		if t["ntp_server"] ~= "" then
			if not is_wp600 then
				result["Setting"]["Setting.cfg"]["Time"]["TimeServer1"] = t["ntp_server"]
			else
				result["__nopath__"][" cfg:/config/system.ini,reboot=1"]["LocalTime.TimeServer1"] = t["ntp_server"]
			end
		end
		-- time zone offset    	
		local timeZone = t["ntp_timezone"]
		if not timeZone or "" == timeZone then
		    timeZone = h.getPbxTimeZone()
		end
		local td, errMsg = require"core.timedate":new(os.time(), timeZone)
		if td then
			td:toTimeZone()
			local tzoffset = td:getTzOffset()
			if tzoffset and tzoffset.sign and tzoffset.hour then
				if not is_wp600 then
        			result["Setting"]["Setting.cfg"]["Time"]["TimeZone"] = tzoffset.sign .. tzoffset.hour
        		else
        			result["__nopath__"][" cfg:/config/system.ini,reboot=1"]["LocalTime.TimeZone"] = tzoffset.signp .. tonumber(tzoffset.hour)
        		end
			end
    	end
    	-- Daylight saving time
        local dst = td and td:getDstData(timeZone)
        if "no" ~= t["use_dst"] and dst then
            if not is_wp600 then
                if is_new_firmware then
                    result["Setting"]["Setting.cfg"]["Time"]["SummerTime"] = "1"
                end
                local wdst = td:getDstByWeek(dst)
                if wdst then
                    result["Setting"]["Setting.cfg"]["Time"]["DSTTimeType"] = "1"
                    result["Setting"]["Setting.cfg"]["Time"]["StartTime"] = wdst[1].month .. "/" .. wdst[1].week .. "/" .. wdst[1].wday .. "/" .. wdst[1].hour
                    result["Setting"]["Setting.cfg"]["Time"]["EndTime"] = wdst[2].month .. "/" .. wdst[2].week .. "/" .. wdst[2].wday .. "/" .. wdst[2].hour
                else
                    result["Setting"]["Setting.cfg"]["Time"]["StartTime"] = dst[1].month .. "/" .. dst[1].day .. "/" .. dst[1].hour
                    result["Setting"]["Setting.cfg"]["Time"]["EndTime"] = dst[2].month .. "/" .. dst[2].day .. "/" .. dst[2].hour
                end
            else
                result["__nopath__"][" cfg:/config/system.ini,reboot=1"]["LocalTime.iDSTType"] = "1" --daylight saving time enabled
                result["__nopath__"][" cfg:/config/system.ini,reboot=1"]["LocalTime.StartTime"] = dst[1].month .. "/" .. dst[1].day .. "/" .. dst[1].hour
                result["__nopath__"][" cfg:/config/system.ini,reboot=1"]["LocalTime.EndTime"] = dst[2].month .. "/" .. dst[2].day .. "/" .. dst[2].hour
            end
        end
	end

	-- Set Date format
	if not is_wp600 and user_features.DATETIME then
		local usa_formats = {["M d Y"]=true, ["m/d/Y"]=true, ["m-d-Y"]=true}
		local phpFormat    = user_features.DATETIME.format
		result["Setting"]["Setting.cfg"]["Time"]["DateFormat"] = usa_formats[phpFormat] and "0" or "6"

		local timeFormat = user_features.DATETIME.is24h
		if timeFormat and "" ~= timeFormat then
			result["Setting"]["Setting.cfg"]["Time"]["TimeFormat"] = timeFormat
		end
	end

	-- Jitter buffer
	local tjb = h.get_jitter_buffer(is_addr_remote)
	if not is_wp600 then
		do local tcfg = result["/"]["vpm.cfg"]["Jitter"]
			tcfg["Adaptive"] = tjb["min"] == tjb["max"] and 0 or 1
			tcfg["Min"] = tjb["min"]
			tcfg["Max"] = tjb["max"]
			tcfg["Nominal"] = tjb["nom"]
			tcfg["Enable"] = 0 -- Important thing to have jb changed with values above
		end
	else
		do local tcfg = result["__nopath__"][" cfg:/config/system.ini,reboot=1"]
			tcfg["Jitter.Adaptive"] = tjb["min"] == tjb["max"] and 0 or 1
			tcfg["Jitter.Min"] = tjb["min"]
			tcfg["Jitter.Max"] = tjb["max"]
			tcfg["Jitter.Nominal"] = tjb["nom"]
			tcfg["Jitter.Enable"] = 0 -- Important thing to have jb changed with values above
		end
	end

	-- TODO: WP600 always reboot if set reboot to 1, need to store somewhere changed laguage or not
	-- Need to recheck, found that phone does not reboot twice
	-- Language
	local language = ({
		it = "Italian",
		de = "Deutsch",
		en = "English",
		fr = "French",
		ru = "Russian",
	})[person["preferredLanguage"]] or "English"
	if not is_wp600 then
		result["Setting"]["Setting.cfg"]["Lang"]["ActiveWebLanguage"] = language
		result["Setting"]["Setting.cfg"]["Lang"]["WebLanguage"] = language
	else
		result["__nopath__"][" cfg:/phone/config/user.ini,reboot=0"]["Lang.ActiveWebLanguage"] = language
		result["__nopath__"][" cfg:/phone/config/user.ini,reboot=0"]["Lang.WEBLanguage"] = language
		result["__nopath__"][" bin:/phone/config/UserLang/lang-".. language .. ".txt,reboot=1"] = {}
		result["__nopath__"][" bin:/phone/config/UserLang/lang-".. language .. ".txt,reboot=1"]["url"] = string.gsub(
			"$protocol://$registrar:$port/?action=getlang&filename=lang-$language.txt", "%$(%w+)",
			{
				protocol  = http_protocol,
				registrar = registrar,
				port      = http_port,
				language  = language
			}
		)
	end
	
	-- Account
    local label = username
    if username:find("unknown%d") then
		label = string.gsub(username, "unknown", "login")
    end
    
    -- Retrieve failover configuration 
    local failover_ip      = nil
    local failover_sipport = nil
    local failoverObj      = require"common.failover":new()
    if failoverObj and "1" == failoverObj:isEnabled() then
        if agent["model"] == "wp400" then
            logger:info("WP400 does not support failover configuration")
            failover_ip   = nil
        else
            local failoverCfg = failoverObj:getConfig()
            failover_ip       = failoverCfg and failoverCfg.pbxFailoverDomain
            failover_sipport  = failoverCfg and failoverCfg.sipPort or "443"
            logger:debug(failoverCfg, "failoverCfg")
        end
    end

	-- ToS parameters
	local tos_data = h.get_sip_settings()
	if not is_wp600 then
		do local tcfg = result["Network"]["Network.cfg"]["QOS"]
		tcfg["RTPTOS"] = tos_data["tos"] / 4 -- Voice QoS
		tcfg["SIGNALTOS"] = tos_data["tos_sip"] / 4 -- SIP QoS
		end
	else
		do local tcfg = result["__nopath__"][" cfg:/config/system.ini,reboot=1"]
		tcfg["QoS.RTPTOS"] = tos_data["tos"] / 4 -- Voice QoS
		tcfg["QoS.SIGNALTOS"] = tos_data["tos_sip"] / 4 -- SIP QoS
		end
	end
	
	-- Processing SIP transport and SIP port
	local gtransp  = nil
	local gsipport = nil
    if is_addr_remote then
        gtransp  = gconf.SIPTransportRemote
		gsipport = gconf.SIPServerPortRemote
    else
        gtransp  = gconf.SIPTransport
		gsipport =  gconf.SIPServerPort
    end
    
    local sip_transport = "TLS"
    if gtransp then
        sip_transport = gtransp
    end

    local srtp    = "0"
    local sipport = "5060"
    if "TLS" == sip_transport then
        srtp         = "1"
        sip_port     = "5061"
        if is_addr_remote then
            sip_port = agent["alternate_port"] or "443"
        end
    end

    if failover_ip then
        sip_transport = "TCP"
        sip_port      = agent["alternate_port"] or "443"
        srtp          = "0"
    end

    if "wp400" == agent["model"] then
        if "TLS" == sip_transport then
            logger:info("WP400 does not support TLS transport. Will be used UDP with SIP port 5060")
            sip_transport = "UDP"
            sip_port      = "5060"
        end
    end
    
    -- Set up registrar data
    sip_transport         = (sip_transport == "TCP" and "1") or (sip_transport == "TLS" and "2") or "0"
	if not is_wp600 then
		do 
			local tcfg                    = result["voip"]["sipAccount0.cfg"]["account"]
			tcfg["AuthName"]              = username
			tcfg["Label"]                 = label
			tcfg["UserName"]              = username
			tcfg["DisplayName"]           = person["cn"] or username
			tcfg["password"]              = secret
			tcfg["SIPServerHost"]         = registrar
			tcfg["OutboundHost"]          = registrar
			tcfg["Transport"]			  = sip_transport
			tcfg["SIPServerPort"]         = sip_port
			tcfg["OutboundPort"]          = sip_port
            tcfg["srtp_encryption"]       = srtp
			if failover_ip then
				result["voip"]["sipAccount1.cfg"] = pbxutils.tcopy(resource.second_account) or {}
				tcfg                          = result["voip"]["sipAccount1.cfg"]["account"]
				tcfg["AuthName"]              = username
				tcfg["Label"]                 = label.."-backup"
				tcfg["UserName"]              = username
				tcfg["DisplayName"]           = person["cn"] or username
				tcfg["password"]              = secret
				tcfg["SIPServerHost"]         = failover_ip
				tcfg["OutboundHost"]          = failover_ip
                tcfg["Transport"]			  = sip_transport
				tcfg["SIPServerPort"]         = failover_sipport
				tcfg["OutboundPort"]          = failover_sipport
                tcfg["srtp_encryption"]       = srtp
			end
		end
	else
		do 
			local tcfg                    = result["__nopath__"][" cfg:/phone/config/voip/sipAccount0.cfg,account=0;reboot=0"]
			tcfg["account.AuthName"]      = username
			tcfg["account.Label"]         = label
			tcfg["account.UserName"]      = username
			tcfg["account.DisplayName"]   = person["cn"] or username
			tcfg["account.password"]      = secret
			tcfg["account.SIPServerHost"] = registrar
			tcfg["account.OutboundHost"]  = registrar
			tcfg["account.SIPServerPort"] = sip_port
			tcfg["account.OutboundPort"]  = sip_port
			tcfg["account.Transport"]	  = sip_transport
            tcfg["account.srtp_encryption"] = srtp
			if failover_ip then
				result["__nopath__"][" cfg:/phone/config/voip/sipAccount1.cfg,account=0;reboot=0"] = pbxutils.tcopy(resource.second_account) or {}
				tcfg                          = result["__nopath__"][" cfg:/phone/config/voip/sipAccount1.cfg,account=0;reboot=0"]
				tcfg["account.AuthName"]      = username
				tcfg["account.Label"]         = label.."-backup"
				tcfg["account.UserName"]      = username
				tcfg["account.DisplayName"]   = person["cn"] or username
				tcfg["account.password"]      = secret
				tcfg["account.SIPServerHost"] = failover_ip
				tcfg["account.OutboundHost"]  = failover_ip
				tcfg["account.SIPServerPort"] = failover_sipport
				tcfg["account.OutboundPort"]  = failover_sipport
                tcfg["account.Transport"]	  = sip_transport
                tcfg["account.srtp_encryption"] = srtp
			end
		end
	end
	
	-- Set user agent
	local useragent = string.format("Wildix %s $FW_VER %s", agent["model"]:upper(), agent["mac"])
	if not is_wp600 then
		result["voip"]["sipAccount0.cfg"]["General"]["UserAgent"] = useragent
		if agent["model"] ~= "wp400" then
			result["voip"]["sipAccount1.cfg"]["General"]["UserAgent"] = useragent
		end
	else
		result["__nopath__"][" cfg:/phone/config/voip/sipAccount0.cfg,account=0;reboot=0"]["General.UserAgent"] = useragent
		result["__nopath__"][" cfg:/phone/config/voip/sipAccount1.cfg,account=0;reboot=0"]["General.UserAgent"] = useragent
	end

	-- Custom admin and user password
	if not is_wp600 then
		result["Setting"]["autop.cfg"]["AdminPassword"]["password"] = token_pass or h.get_gpassword()
		result["Setting"]["autop.cfg"]["UserPassword"]["password"] = token_pass or h.get_gpassword()
		result["Setting"]["autop.cfg"]["VarPassword"]["password"] = token_pass or h.get_gpassword()
	else
		result["__nopath__"][" psw:/phone/config/.htpasswd"]["admin"] = token_pass or h.get_gpassword()
		result["__nopath__"][" psw:/phone/config/.htpasswd"]["user"] = token_pass or h.get_gpassword() -- NOTE: impossible to remove user "user" currently
	end

	-- Ringtones (Must be NULL. We use ring tones from PBX)
	do
		local tcfg = (is_wp600 and result["__nopath__"][" cfg:/phone/config/user.ini,reboot=0"] or result["Setting"]["Setting.cfg"])
		for i=1,10 do		-- WP has 10 fields for Ringtones
			local k = "AlertInfo" .. tostring(i-1)
			if not is_wp600 then
				tcfg[k]["Text"] = "%NULL%"
				tcfg[k]["Ringer"] = 1
			else
				tcfg[k .. ".Text"] = ""
				tcfg[k .. ".Ringer"] = 1
			end
		end
	end

	-- Inter digit time and voice mail access
	local inter_digit_time = h.get_interdigit_timeout()
	local vmnumber = user_features["SHAREDVM"] and user_features["SHAREDVM"].status == "1" and user_features["SHAREDVM"].value ~= "" and user_features["SHAREDVM"].value or username
	if not is_wp600 then
		result["Features"]["Message.cfg"]["Message"]["VoiceNumber0"] = "vmaccess*" .. vmnumber
		result["Setting"]["Setting.cfg"]["PhoneSetting"]["InterDigitTime"] = inter_digit_time
	else
		result["__nopath__"][" cfg:/phone/config/user.ini,reboot=0"]["PhoneSetting.InterDigitTime"] = inter_digit_time
		result["__nopath__"][" cfg:/phone/config/user.ini,reboot=0"]["Message.VoiceNumber0"] = "vmaccess*" .. vmnumber
	end

	-- Provision url
	local provlink = string.gsub(
		"$protocol://$registrar:$port/?action=setaps", "%$(%w+)",
		{
			registrar = registrar,
			protocol  = http_protocol,
			port      = http_port,
		}
	)
	if not is_wp600 then
		do
			local tcfg = result["Setting"]["autop.cfg"]["autoprovision"]
			tcfg["server_address"] = provlink
			tcfg["user"] = "wmsdevice"
			tcfg["password"] = token_pass or h.get_gpassword()
		end
	else
		do
			local tcfg = result["__nopath__"][" cfg:/config/system.ini,reboot=1"]
			tcfg["AutoProvision.strServerURL"] = provlink
			tcfg["AutoProvision.strUser"] = "wmsdevice"
			tcfg["AutoProvision.strPassword"] = token_pass or h.get_gpassword()
		end
	end

	-- End of digit
	local eod = h.get_eod()
	if eod == "1" then
		eod = "2"
	elseif eod == "2" then
		eod = "1"
	end

	-- Static features for new firmware
	if is_newV61_firmware then
		result["Features"]["Phone.cfg"]["SRemoteName"] = {}
		result["Features"]["Phone.cfg"]["SRemoteName"]["Enable"] = "0"
		result["Features"]["Phone.cfg"]["Features"]["AutoAnswerDelay"] = "0"
		result["Features"]["Phone.cfg"]["Features"]["Refuse_Code"] = "603"
		result["Features"]["Forward.cfg"]["TimeoutFWD"] = {}
		result["Features"]["Forward.cfg"]["TimeoutFWD"]["Timeout"] = "10"

		result["Setting"]["AdvSetting.cfg"]["Transfer"] = {}
		result["Setting"]["AdvSetting.cfg"]["Transfer"]["DSSKeyDealType"] = gconf["BLFTransType"] or "1"
		result["Setting"]["AdvSetting.cfg"]["Transfer"]["EnableSemiAttendTran"] = "1"
	end

	if is_new_firmware then
		result["Features"]["Phone.cfg"]["Features"]["HideDTMF"] = "0"
		result["Features"]["Phone.cfg"]["Features"]["HideDTMFDelay"] = "0"
		result["Features"]["Phone.cfg"]["Features"]["SaveCallHistory"] = "1"
		result["Features"]["Phone.cfg"]["Features"]["ButtonSoundOn"] = h.get_button_sound()
		result["Features"]["Phone.cfg"]["Features"]["BroadsoftFeatureKeySync"] = "1"
		result["Features"]["Phone.cfg"]["PoundSend"] = {}
		result["Features"]["Phone.cfg"]["PoundSend"]["Enable"] = eod
		result["Setting"]["Setting.cfg"]["PhoneSetting"]["LogonWizard"] = "0"
		result["Setting"]["autop.cfg"]["PNP"] = {}
		result["Setting"]["autop.cfg"]["PNP"]["Pnp"] = "0"
		result["Advanced"]["Advanced.cfg"] = {}
		result["Advanced"]["Advanced.cfg"]["Zero"] = {}
		result["Advanced"]["Advanced.cfg"]["Zero"]["ForbidZero"] = "1"
	elseif is_wp600 then
		result["__nopath__"][" cfg:/phone/config/user.ini,reboot=0"]["Features.ButtonSoundOn"] = h.get_button_sound()
		result["__nopath__"][" cfg:/phone/config/user.ini,reboot=0"]["PoundSend.Enable"] = eod
		if is_v21_wp600_fmw then
			result["__nopath__"][" cfg:/phone/config/user.ini,reboot=0"]["Transfer.DSSKeyDealType"] = gconf["BLFTransType"] or "1"
		end
	end
	
	-- Codecs right order for corresponding network band
	local is_wideband = h.is_wideband(agent["address"])
	if not is_wp600 then
		do
			local netband_str = is_wideband and "wideband" or "narrowband"
			local codecs = resource.codecs[netband_str]
			local tcfg = result["voip"]["sipAccount0.cfg"]
			for key, value in pairs(codecs) do
				if agent["model"] == "wp400" and value["PayloadType"] == "G722" then
					value["enable"] = "0"	-- WP400 don't have correct support G722 disable it
				end
				tcfg[key] = value
			end
			-- set nerrowband codecs for failover registration
			if failover_ip then
				tcfg = result["voip"]["sipAccount1.cfg"]
				if netband_str ~= "narrowband" then
					codecs = resource.codecs["narrowband"]
				end
				for key, value in pairs(codecs) do
					if agent["model"] == "wp400" and value["PayloadType"] == "G722" then
						value["enable"] = "0"	-- WP400 don't have correct support G722 disable it
					end
					tcfg[key] = value
				end
			end
			if is_codec_priority then
				result["voip"]["sipAccount0.cfg"]["account"]["FullSDPAnswer"] = 1
				if netband_str == "wideband" then
					result["voip"]["sipAccount0.cfg"]["account"]["UseLocalCodecPriority"] = "0"
				else
					result["voip"]["sipAccount0.cfg"]["account"]["UseLocalCodecPriority"] = "1"
				end
				if failover_ip then
					result["voip"]["sipAccount1.cfg"]["account"]["FullSDPAnswer"] = 1
					result["voip"]["sipAccount1.cfg"]["account"]["UseLocalCodecPriority"] = "1"
				end
			end
		end
	else
		do
			local netband_str = is_wideband and "wideband" or "narrowband"
			local codecs = resource.codecs[netband_str]
			local tcfg = result["__nopath__"][" cfg:/phone/config/voip/sipAccount0.cfg,account=0;reboot=0"]
			for key, value in pairs(codecs) do
				for key_suffix, final_value in pairs(value) do
					tcfg[key .. "." .. key_suffix] = final_value
				end
			end
			-- set nerrowband codecs for failover registration
			if failover_ip then
				tcfg = result["__nopath__"][" cfg:/phone/config/voip/sipAccount1.cfg,account=0;reboot=0"]
				if netband_str ~= "narrowband" then
					codecs = resource.codecs["narrowband"]
				end
				for key, value in pairs(codecs) do
					for key_suffix, final_value in pairs(value) do
						tcfg[key .. "." .. key_suffix] = final_value
					end
				end
			end
			if is_codec_priority then
				result["__nopath__"][" cfg:/phone/config/voip/sipAccount0.cfg,account=0;reboot=0"]["account.FullSDPAnswer"] = 1
				if netband_str == "wideband" then
					result["__nopath__"][" cfg:/phone/config/voip/sipAccount0.cfg,account=0;reboot=0"]["account.UseLocalCodecPriority"] = "0"
				else
					result["__nopath__"][" cfg:/phone/config/voip/sipAccount0.cfg,account=0;reboot=0"]["account.UseLocalCodecPriority"] = "1"
				end
				if failover_ip then
					result["__nopath__"][" cfg:/phone/config/voip/sipAccount1.cfg,account=0;reboot=0"]["account.FullSDPAnswer"] = 1
					result["__nopath__"][" cfg:/phone/config/voip/sipAccount1.cfg,account=0;reboot=0"]["account.UseLocalCodecPriority"] = "1"
				end
			end
		end
	end

	-- LogoF and dialnow url
	local is_logo_used = is_wildix_firmware and h.is_exists("/var/www/images/logo.dob")
	if is_logo_used then
		-- Phone.cfg:LCD_Logo = 2 # 1 - system, 2 - custom
		result["Features"]["Phone.cfg"]["Features"]["LCD_Logo"] = "2"
	end
	if is_wp600 then
		local serverlink = string.gsub(
			"$protocol://$registrar:$port/", "%$(%w+)",
			{
				protocol  = http_protocol,
				registrar = registrar,
				port      = http_port,
			}
		)
		result["__nopath__"][" bin:/phone/userdata/ScreenSaver/wp600wallpaper.jpg"]["url"] = serverlink .. "?action=getscreensaver"
		result["__nopath__"][" bin:/phone/userdata/BackGround/wp600wallpaper.jpg"]["url"] = serverlink .. "?action=getwallpaper"
		result["__nopath__"][" bin:/phone/config/DialNow.xml,reboot=1"]["url"] = serverlink .. "?action=getdialnowpatters"
	end
	-- Disable logo for phones except wp500 to avoide starting screensaver
	if agent["model"] ~= "wp500" and not is_wp600 then
		result["Features"]["Phone.cfg"]["Features"]["LCD_Logo"] = "0"
	end

	-- Multicast paging
	if is_newV61_firmware or is_v14_wp600_fmw then
		if connection:switch("wmsdb") then
			local sql = "SELECT p.id, p.name FROM paging AS p LEFT JOIN paging_members AS pm ON (pm.paging_id = p.id) WHERE pm.group_member = %q order by p.id limit 10"
			local res_sql = connection:rows (sql, username)
			if res_sql then -- check: is the tables for paging exist
				if is_newV61_firmware then
					result["vpPhone"]["MulticastPage.cfg"] = {}
					result["vpPhone"]["MulticastPage.cfg"]["audio"] = {}
					result["vpPhone"]["MulticastPage.cfg"]["audio"]["PayloadType"] = "PCMA"
					result["vpPhone"]["MulticastPage.cfg"]["ReceivePriority"] = {}
					result["vpPhone"]["MulticastPage.cfg"]["ReceivePriority"]["Active"] = "0"
					result["vpPhone"]["MulticastPage.cfg"]["ReceivePriority"]["Priority"] = "0"
					
					for n = 1, 10 do	-- filling default value
						result["vpPhone"]["MulticastPage.cfg"]["ListenAddress"..n] = {}
						result["vpPhone"]["MulticastPage.cfg"]["ListenAddress"..n]["Label"] = "NoGroup"
						result["vpPhone"]["MulticastPage.cfg"]["ListenAddress"..n]["IPAddress"] = "224.0.3.0:999"..(n - 1)
					end
				
					local i = 1
					for id, name in res_sql do	-- filling value from table paging
						result["vpPhone"]["MulticastPage.cfg"]["ListenAddress"..i]["Label"] = name
						result["vpPhone"]["MulticastPage.cfg"]["ListenAddress"..i]["IPAddress"] = "239.1.1.10:" .. tostring(9999+id)
						i = i + 1
					end
				elseif is_v14_wp600_fmw then
			 		result["__nopath__"][" cfg:/phone/config/vpPhone/MulticastPage.cfg,reboot=1"] = {}
					result["__nopath__"][" cfg:/phone/config/vpPhone/MulticastPage.cfg,reboot=1"]["audio.PayloadType"] = "PCMA"
					result["__nopath__"][" cfg:/phone/config/vpPhone/MulticastPage.cfg,reboot=1"]["ReceivePriority.Active"] = "0"
					result["__nopath__"][" cfg:/phone/config/vpPhone/MulticastPage.cfg,reboot=1"]["ReceivePriority.Priority"] = "0"
					
					for n = 1, 10 do	-- filling default value
						result["__nopath__"][" cfg:/phone/config/vpPhone/MulticastPage.cfg,reboot=1"]["ListenAddress".. n ..".Label"] = "NoGroup"
						result["__nopath__"][" cfg:/phone/config/vpPhone/MulticastPage.cfg,reboot=1"]["ListenAddress".. n ..".IPAddress"] = "224.0.3.0:999"..(n - 1)
					end
		
					local i = 1
					for id, name in res_sql do	-- filling value from table paging
						result["__nopath__"][" cfg:/phone/config/vpPhone/MulticastPage.cfg,reboot=1"]["ListenAddress".. i ..".Label"] = name
						result["__nopath__"][" cfg:/phone/config/vpPhone/MulticastPage.cfg,reboot=1"]["ListenAddress".. i ..".IPAddress"] = "239.1.1.10:" .. tostring(9999+id)
						i = i + 1
					end
				end
			end
			connection:close()
		end
	end
	
	-- Make output
	--local config_path = is_wildix_firmware and "wildix" or "yealink"
	local config_path = "yealink"
	local output = {}
	for k,v in pairs (result) do
		if k ~= "__nopath__" then
			for filename, _ in pairs(v) do
				for key, _ in pairs(v[filename]) do
					local parameters = v[filename][key]
					output[#output+1] = "[".. key .."]"
					if k == "/" then
						output[#output+1] = "path=/" .. config_path .. "/config/" .. filename
					else
						output[#output+1] = "path=/" .. config_path .. "/config/" .. k .. "/" .. filename
					end

					for param, value in pairs(parameters) do
						output[#output+1] = param .. "=" .. value
					end
				end
			end
		else
			for key, _ in pairs(v) do
				output[#output+1] = "[".. key .."]"
				for param, value in pairs(v[key]) do
					output[#output+1] = param .. "=" .. value
				end
			end
		end
	end

	-- Language file
	if not is_wp600 then
		output[#output+1] = "[Lang]"
		output[#output+1] = "path=/tmp/y00000000000" .. tostring(({
			["wp400"] = "9",
			["wp480"] = "5",
			["wp490"] = "4",
			["wp500"] = "0"
		})[agent["model"]]) .. ".cfg"
		output[#output+1] = string.gsub(
			"server_address=$protocol://$registrar:$port/?action=getlang&filename=lang-$language.txt", "%$(%w+)",
			{
				protocol  = http_protocol,
				registrar = registrar,
				port      = http_port,
				language  = language
			}
		)
	end

	-- Logo
	if is_logo_used and not is_wp600 then
		output[#output+1] = "[Logo]"
		output[#output+1] = "path=/wildix/config/Pics/logo.dob"
		output[#output+1] = string.gsub(
			"server_address=$protocol://$registrar:$port/?action=getlogo&filename=logo.dob", "%$(%w+)",
			{
				protocol  = http_protocol,
				registrar = registrar,
				port      = http_port
			}
		)
	end

	-- TODO
	-- // + /yealink/config/DialRule/dialnow.xml
	connection:switch ("devdb")
	local sql = "SELECT value FROM system WHERE variable=%q"
	local cursor = connection:execute (sql, "QUICK_DIAL_PATTERNS")
	local dpatterns = cursor:fetch(); cursor:close()
	connection:close()
	if not is_wp600 then
		output[#output+1] = "[DialNow]"
		output[#output+1] = "path=/tmp/dialnow.xml"
		if dpatterns and dpatterns ~= "" and agent["model"] ~= "wp400" then
			local i = 1
			for patt in dpatterns:gmatch("([^%|]*)|?") do
				if patt ~= "" then
					output[#output+1] = i .. "=" .. patt; i = i + 1
				end
			end
		else
			for i=1, 10 do
				output[#output+1] = i .. "=*" -- erase
			end
		end
	end
	return output, username, err
end

function setuser (host, port, ...)
-- expect username & password as arguments
-- need to get from feature database the F_KEYS
	local args = select(1, ...)
	if args["username"] then
	    local core     = require"common.core"
	    local h        = require"common.helper"
	    local pbxutils = require"core.pbxutils"
	    local http     = require"core.httpClient":new()
		local connection = core.connection
		local json = require"dkjson"
		
		local siplistenport = 28034
		
		-- processing registrar address
        local registrar       = nil
        local case            = "hosts"
        local cfg             = {
                FailoverByDns    = gconf.FailoverByDns
               ,ForceResolveName = gconf.ForceResolveName
               ,ForceResolveIp   = gconf.ForceResolveIp
              }
    	local addr_t, err_msg = h.resolve_by_hosts(host.ip_src, nil, cfg)
    	if not addr_t then
    		if err_msg then	-- printing resolving error
    			logger:warning("Hosts file was not processed: %s", err_msg)
    		end
    	else
    		registrar = addr_t.domain
    	end
    	if not registrar then
    		registrar, case, err_msg = h.get_registrar(host.ip_src, host.ip) 
    	end
    	logger:debug("Registrar address %q got from %s%s", registrar, case, err_msg and ". Errors: "..err_msg or "")

		local agent = {
			["vendor"]         = host["vendor"]:lower(),
			["mac"]            = host["mac_addr_str"],
			["registrar"]      = registrar,
			["registrar:port"] = "80",
			["fw:version"]     = host.fwversion,
			["ssl"]            = "off",
			["model"]          = host.model,
			["address"]        = host.ip
		}
		local secret = require"base64".decode(args["password"])
		local label = args["username"]
		if label:find("unknown%d") then
			label = string.gsub(label, "unknown", "login")
        end
            
        -- Reading failover configuration
        local failover_ip      = nil
        local failover_sipport = nil
        local failoverObj      = require"common.failover":new()
        if failoverObj and "1" == failoverObj:isEnabled() then
            if agent["model"] == "wp400" then
                logger:info("WP400 does not support failover configuration")
                failover_ip   = nil
            else
                local failoverCfg = failoverObj:getConfig()
                failover_ip       = failoverCfg and failoverCfg.pbxFailoverDomain
                failover_sipport  = failoverCfg and failoverCfg.sipPort or "443"
                logger:debug(failoverCfg, "failoverCfg")
            end
        end

		-- Processing SIP transport and SIP port
        local is_addr_remote = not h.isLocalIP(agent["address"])
        local gtransp  = nil
        local gsipport = nil
        if is_addr_remote then
            gtransp  = gconf.SIPTransportRemote
            gsipport = gconf.SIPServerPortRemote
        else
            gtransp  = gconf.SIPTransport
            gsipport =  gconf.SIPServerPort
        end

        local sip_transport = "TLS"
        if gtransp then
            sip_transport = gtransp
        end

        local srtp       = "0"
        local sip_port   = "5060"
        if "TLS" == sip_transport then
            srtp         = "1"
            sip_port     = "5061"
            if is_addr_remote then
                sip_port = agent["alternate_port"] or "443"
            end
        end

        if failover_ip then
            sip_transport = "TCP"
            sip_port      = agent["alternate_port"] or "443"
            srtp          = "0"
        end

        if "wp400" == agent["model"] then
            if "TLS" == sip_transport then
                logger:info("WP400 does not support TLS transport. Will be used UDP with SIP port 5060")
                sip_transport = "UDP"
                sip_port      = "5060"
                srtp          = "0"
            end
        end

		sip_transport = (sip_transport == "TCP" and "1") or (sip_transport == "TLS" and "2") or "0"

		local t = {
			username      = args["username"],
			label         = label,
			secret        = http.urlEncode(secret),
			registrar     = registrar,
			device        = json.encode(agent),
			srtp          = srtp,
			transport     = sip_transport,
			sipport	      = sip_port,
			siplistenport = siplistenport
		}
		
		-- Codecs right order for corresponding network band
		local is_wideband = h.is_wideband (host.ip)
		local netband_str = is_wideband and "wideband" or "narrowband"
		local resource = host["model"] ~= "wp600" and require"resources.wildixp" or require"resources.wp600"
		local codecs = resource.codecs[netband_str]
		for s, p in pairs(codecs) do
			if agent["model"] == "wp400" and p["PayloadType"] == "G722" then
				p["enable"] = "0"	-- WP400 don't have correct support G722 disable it
			end
			if tonumber(p["priority"]) > 0 and tonumber(p["enable"]) > 0 then
				t["codec" .. p["priority"]] = p["PayloadType"]
			end
		end
		
		-- Set failover account configuration
		local ft = nil
		if failover_ip then
			if netband_str == "narrowband" then
				ft = pbxutils.tcopy(t)
				ft.registrar = failover_ip
				ft.label     = label.."-backup"
				ft.srtp      = srtp
				ft.transport = sip_transport
				ft.sipport   = failover_sipport
			else
				ft = {
					username  = args["username"],
					label     = label.."-backup",
					secret    = http.urlEncode(secret),
					registrar = failover_ip,
					device    = json.encode(agent),
					srtp      = srtp,
					transport = sip_transport,		
					sipport   = failover_sipport,
					siplistenport = siplistenport
				}
				for s, p in pairs(resource.codecs["narrowband"]) do
					if agent["model"] == "wp400" and p["PayloadType"] == "G722" then
						p["enable"] = "0"	-- WP400 don't have correct support G722 disable it
					end
					if tonumber(p["priority"]) > 0 and tonumber(p["enable"]) > 0 then
						ft["codec" .. p["priority"]] = p["PayloadType"]
					end
				end
			end
		end
		
		local sql = "INSERT OR REPLACE INTO autoprov (id, param, value, mac, token) VALUES((SELECT id FROM autoprov WHERE param=%q AND mac=%q), %q, '%s', %q, %q)"
		connection:execute (sql, "device", host.mac_addr_str, "device", t["device"], host.mac_addr_str, host.token or '')

		-- Get http proto
		local is_wp600 = host.model == "wp600"
		local PORT_NUMBER   = host.ssl and not gconf.HTTPForceUse and 443 or 80
		local http_protocol = (host.model == "wp400" or gconf.HTTPForceUse) and "http" or "https"
		local http_port 	= (host.model == "wp400" or gconf.HTTPForceUse) and "80" or "443"
		
		local devFwInfo 	  = h.parse_firmware(host.fwversion)
		local model_spec	  = devFwInfo and devFwInfo["model"]
		local major_version	  = devFwInfo and devFwInfo["major"]
		local is_new_firmware = model_spec and tonumber(major_version) >= 60 and true or false
		
		-- Set username and password, see order on the page (note order is important)
		logger:info("Configuration updates by setting username to %q at %s", t.username or "", host.model or "")
		
		local options = { authUser = host.auth.user, authPass = host.auth.pass }

		-- Set voicemail number
		t["vmnumber"] = t["username"]
		local user_features
		local uid = args["userid"]
		if uid then
			user_features = require"common.userFeatures.userFeatures":load(uid)
			if user_features["SHAREDVM"] and user_features["SHAREDVM"].status == "1" and user_features["SHAREDVM"].value and user_features["SHAREDVM"].value ~= "" then
				t["vmnumber"] = user_features["SHAREDVM"].value
			end
		end
		
		local is_reachable, is_reachable2, data, config_data, answer = false, false
		local touts = {5000, 7000, 10000}
		for i=1, #touts do
			options.timeout = touts[i]
			if is_wp600 then
				-- Set up account configuration 
				config_data = {
					http.urlEncode('profileOpen("/phone/config/voip/sipAccount0.cfg")'),
					http.urlEncode('profileSetString("account","Enable","1")'),
					http.urlEncode('profileSetString("account","Label","' .. t['label'] .. '")'),
					http.urlEncode('profileSetString("account","DisplayName","' .. t['username'] .. '")'),
					http.urlEncode('profileSetString("account","UserName","' .. t['username'] .. '")'),
					http.urlEncode('profileSetString("account","AuthName","' .. t['username'] .. '")'),
					http.urlEncode('profileSetString("account","SIPServerHost","' .. t['registrar'] .. '")'),
					http.urlEncode('profileSetString("account","SIPServerPort","' .. t['sipport'] .. '")'),
					http.urlEncode('profileSetString("account","OutboundHost","' .. t['registrar'] .. '")'),
					http.urlEncode('profileSetString("account","OutboundPort","' .. t['sipport'] .. '")'),
					http.urlEncode('profileSetString("account","SIPListenPort","' .. t['siplistenport'] .. '")'),
					http.urlEncode('profileSave()'),
					http.urlEncode('profileOpen("/phone/config/user.ini")'),
					http.urlEncode('profileSetString("Message","VoiceNumber0","vmaccess*' .. t['vmnumber'] .. '")'),
					http.urlEncode('profileSave()'),
					http.urlEncode('regSetPasswdString("/phone/config/voip/sipAccount0.cfg","account","password","') .. t['secret'] .. http.urlEncode('")'),
					http.urlEncode('msgBroadcastPostMessage("0x10007","3","0")'),
					http.urlEncode('profileOpen("/phone/config/voip/sipAccount1.cfg")'),
				}
				if failover_ip then
					table.insert(config_data, http.urlEncode('profileSetString("account","Enable","1")'))
					table.insert(config_data, http.urlEncode('profileSetString("account","Label","' .. ft['label'] .. '")'))
					table.insert(config_data, http.urlEncode('profileSetString("account","DisplayName","' .. ft['username'] .. '")'))
					table.insert(config_data, http.urlEncode('profileSetString("account","UserName","' .. ft['username'] .. '")'))
					table.insert(config_data, http.urlEncode('profileSetString("account","AuthName","' .. ft['username'] .. '")'))
					table.insert(config_data, http.urlEncode('profileSetString("account","SIPServerHost","' .. ft['registrar'] .. '")'))
					table.insert(config_data, http.urlEncode('profileSetString("account","SIPServerPort","' .. ft['sipport'] .. '")'))
					table.insert(config_data, http.urlEncode('profileSetString("account","OutboundHost","' .. ft['registrar'] .. '")'))
					table.insert(config_data, http.urlEncode('profileSetString("account","OutboundPort","' .. ft['sipport'] .. '")'))
					table.insert(config_data, http.urlEncode('profileSetString("account","SIPListenPort","' .. ft['siplistenport'] .. '")'))
					table.insert(config_data, http.urlEncode('profileSave()'))
					table.insert(config_data, http.urlEncode('regSetPasswdString("/phone/config/voip/sipAccount1.cfg","account","password","') .. ft['secret'] .. http.urlEncode('")'))
					logger:debug("Prepare failover account parameters")
				else
					table.insert(config_data, http.urlEncode('profileSetString("account","Enable","0")'))
					table.insert(config_data, http.urlEncode('profileSetString("account","Label","")'))
					table.insert(config_data, http.urlEncode('profileSetString("account","DisplayName","")'))
					table.insert(config_data, http.urlEncode('profileSetString("account","UserName","")'))
					table.insert(config_data, http.urlEncode('profileSetString("account","AuthName","")'))
					table.insert(config_data, http.urlEncode('profileSetString("account","SIPServerHost","")'))
					table.insert(config_data, http.urlEncode('profileSetString("account","SIPServerPort","5060")'))
					table.insert(config_data, http.urlEncode('profileSetString("account","OutboundHost","")'))
					table.insert(config_data, http.urlEncode('profileSetString("account","OutboundPort","5060")'))
					table.insert(config_data, http.urlEncode('profileSave()'))
					table.insert(config_data, http.urlEncode('regSetPasswdString("/phone/config/voip/sipAccount1.cfg","account","password","")'))
					logger:debug("Set failover account as disable")
				end
				table.insert(config_data, http.urlEncode('msgBroadcastPostMessage("0x10007","3","1")'))
				
				data = "postCommand=" .. table.concat(config_data)
				answer = http:rawpost(host.ip, PORT_NUMBER, "/cgi-bin/cgiServer.exx", data, options) or {}
				logger:debug("Setting accounts parameters for %s", host.model, answer.code or 408)
				is_reachable = true
				is_reachable2 = true
			else
				local str
				-- main account
				if not is_reachable then
					if is_new_firmware then
						str = "PAGEID=4&CONFIG_DATA=0%26$username%26$username%26$username%26$secret%26$registrar%26$sipport%261%26$registrar%26$sipport%260%26%265060%261%26$codec1%26$codec2%26$codec3%26$codec4%26$codec5%26%26%26%26%26%26%26%26%26300%261%2630%2611780%2612780%260%26$siplistenport%26217.10.79.21%2610000%260.5%264%265%26vmaccess*$vmnumber%26%261200%26%261200%262%261%26101%260%260%260%261%260%267200%260%260%26$srtp%261%2620%26%261%260%260%26$label%26$transport%260%261%261%260%26%26300%263600%260%260%2630%26%26%26%26%26common%260%26%260%26%263600%26%260%260%261"
					else
						str = "PAGEID=4&CONFIG_DATA=0%26$username%26$username%26$username%26$secret%26$registrar%26$sipport%261%26$registrar%26$sipport%260%26%265060%261%26$codec1%26$codec2%26$codec3%26$codec4%26$codec5%26%26%26%26%26%26%26300%261%2630%2611780%2612780%260%265060%26217.10.79.21%2610000%260.5%264%265%26vmaccess*$vmnumber%26%265060%26%261200%262%261%26101%260%260%260%260%260%26%260%260%260%260%2620%26%261%260%260%26$label%260%260%260%261"
					end
					data = string.gsub(str, "%$(%w+)", t)
					answer = http:rawpost(host.ip, PORT_NUMBER, "/cgi-bin/ConfigManApp.com", data, options) or {}
					logger:debug("Set account parameters for %s - %s", host.model, answer.code or 408)
					is_reachable = answer.body:find("SIP PHONE") ~= nil or answer.body:find("IP Phone") ~= nil
				end
				-- failover account
				if is_new_firmware and not is_reachable2 then
					if failover_ip then
						str = "PAGEID=4&CONFIG_DATA=1%26$username%26$username%26$username%26$secret%26$registrar%26$sipport%261%26$registrar%26$sipport%260%26%265060%261%26$codec1%26$codec2%26$codec3%26$codec4%26$codec5%26%26%26%26%26%26%26%26%26300%261%2630%2611780%2612780%260%26$siplistenport%26217.10.79.21%2610000%260.5%264%265%26%26%261200%26%261200%262%261%26101%260%260%260%261%260%267200%260%260%26$srtp%261%2620%26%261%260%260%26$label%26$transport%260%261%261%260%26%26300%263600%260%260%2630%26%26%26%26%26common%260%26%260%26%263600%26%260%260%261"
						data = string.gsub(str, "%$(%w+)", ft)
						answer = http:rawpost(host.ip, PORT_NUMBER, "/cgi-bin/ConfigManApp.com", data, options) or {}
						logger:debug("Set failover account parameters for %s - %s", host.model, answer.code or 408)
					elseif agent["model"] ~= "wp400" then
						data = "PAGEID=4&CONFIG_DATA=1%26%26%26%26%26%265060%261%26%265060%260%26%265060%260%26G729%26%26%26%26%26%26%26%26%26%26%26%26%26300%261%2630%2611780%2612780%260%265060%26217.10.79.21%2610000%260.5%264%265%26%26%261200%26%261200%262%261%26101%260%260%260%261%260%267200%260%260%26%261%2620%26%261%260%260%26%26%260%261%261%260%26%26300%263600%260%260%2630%26%26%26%26%26common%260%26%260%26%263600%26%260%260%261"
						answer = http:rawpost(host.ip, PORT_NUMBER, "/cgi-bin/ConfigManApp.com", data, options) or {}
						logger:debug("Set failover account as disable for %s - %s", host.model, answer.code or 408)
					end
					if agent["model"] ~= "wp400" then
						is_reachable2 = answer.body:find("SIP PHONE") ~= nil or answer.body:find("IP Phone") ~= nil
					else
						is_reachable2 = true
					end
				end
			end
			if is_reachable and is_reachable2 then break end
			require"socket".sleep(1)
		end
		if not is_reachable and not is_reachable2 then
			logger:error("Device %s(%s) is unreachable", host.model or "", host.ip or "")
			return false
		end
		
		-- Get security password
		local gpassword
		if host.token_type == "new" then
			gpassword = host.token or h.get_gpassword()
		else
			gpassword = h.get_gpassword()
			if host.token_type == "old" then
				logger:info("Token has old format")
			end
		end

		-- Provisioning access
		local provlink = string.gsub(
			"$protocol://$registrar:$port/?action=setaps", "%$(%w+)",
			{
				registrar = t["registrar"],
				protocol  = http_protocol,
				port      = http_port,
			}
		)

		if is_wp600 then
			-- Set up provisioning configuration
			config_data = {
				http.urlEncode('profileOpen("/phone/config/system.ini")'),
				http.urlEncode('profileSetString("AutoProvision","bEnablePNP","0")'),
				http.urlEncode('profileSetString("AutoProvision","bEnableDHCPOption","0")'),
				http.urlEncode('profileSetString("AutoProvision","listUserOptions","128")'),
				http.urlEncode('profileSetString("AutoProvision","nUserOptionsType","1")'),
				http.urlEncode('profileSetString("AutoProvision","strServerURL","' .. provlink .. '")'),
				http.urlEncode('profileSetString("AutoProvision","strUser","wmsdevice")'),
				http.urlEncode('profileSetString("AutoProvision","strPassword",' .. gpassword .. ')'),
				http.urlEncode('profileSetString("AutoProvision","bEnablePowerOn","1")'),
				http.urlEncode('profileSetString("AutoProvision","bEnableRepeat","0")'),
				http.urlEncode('profileSetString("AutoProvision","bEnableWeekly","0")'),
				http.urlEncode('profileSave()'),
			}
			data = "postCommand=" .. table.concat(config_data)
			answer = http:rawpost(host.ip, PORT_NUMBER, "/cgi-bin/cgiServer.exx", data, options) or {}
			logger:debug("Updating provisioning parameters for %s", host.model, answer.code or 408)
			
			-- Run autoprovisioning to get features and other options 
			data = "postCommand=" .. http.urlEncode('msgSendMessage("autoServer","0xA0000","0","0")')
			answer = http:rawpost(host.ip, PORT_NUMBER, "/cgi-bin/cgiServer.exx", data, options) or {}
			logger:info("Run autoprovisioning for set username at %s", host.model, answer.code or 408)
		else
			-- get device settings for check syslog_server
			local settings = {}
			local settings_json = get_settings(agent["mac"])
			if settings_json then
				settings = json.decode(settings_json)
			end
			local syslog_level = "6"
			local syslog_server = settings["syslog_server"] or ""
			config_data = {"128", "1", provlink, "wmsdevice", gpassword, syslog_server, "", "1", "", "00:00", "00:00", "", "", "0", "1", "", syslog_level}
			data = "PAGEID=16&CONFIG_DATA=" .. http.urlEncode(table.concat(config_data, "þ"))
			answer = http:rawpost(host.ip, PORT_NUMBER, "/cgi-bin/ConfigManApp.com", data, options) or {}
			logger:debug("Updating provisioning parameters for %s - %s", host.model, answer.code or 408)

			-- Set up function Keys
			if not args["username"]:find("unknown%d") and uid then
				local keys = core.fkeys_get(user_features, args["username"])
				keys = convert(keys, "html", host.model, is_new_firmware, is_wp600)
	
				if #keys["lineKeys"] > 0 then
					config_data = table.concat(keys["lineKeys"], "&")
					data = "PAGEID=20&CONFIG_DATA=" .. http.urlEncode(config_data)
					answer = http:rawpost(host.ip, PORT_NUMBER, "/cgi-bin/ConfigManApp.com", data, options) or {}
					logger:debug("Updating DSS line keys for %s - %s", host.model, answer.code or 408)
				end
	
				if #keys["stdKeys"] > 0 then
					config_data = table.concat(keys["stdKeys"], "&")
					data = "PAGEID=21&CONFIG_DATA=" .. http.urlEncode(config_data)
					answer = http:rawpost(host.ip, PORT_NUMBER, "/cgi-bin/ConfigManApp.com", data, options) or {}
					logger:debug("Updating DSS memory keys for %s - %s", host.model, answer.code or 408)
				end
	
				for serial, t in pairs(keys["expKeys"]) do
					if #t > 0 then
						config_data = serial .. "&" .. table.concat (t, "&")
						data = "PAGEID=31&CONFIG_DATA=" .. http.urlEncode(config_data)
						answer = http:rawpost(host, PORT_NUMBER, "/cgi-bin/ConfigManApp.com", data, options) or {}
						logger:debug("Updating EXP module keys for %s - %s", host.model, answer.code or 408)
					end
				end
			end
	
			-- Features
			if not label:find("login%d") then
				setfeatures(host, port, ...)
			end
		end
	end
end

function setfeatures (host, port, ...)
	local args = select(1, ...)
	
	local core    = require"common.core"
	local h       = require"common.helper"
	local wmsldap = require"core.wmsldap"
	local http    = require"core.httpClient":new()

	local uid = args["userid"] or ""
	local feature = args["feature"]
	local user_features = require"common.userFeatures.userFeatures":load(uid)
	local PORT_NUMBER = host.ssl and not gconf.HTTPForceUse and 443 or 80
	
	-- processing registrar address
	local registrar       = nil
	local case            = "hosts"
    local cfg             = {
                FailoverByDns    = gconf.FailoverByDns
               ,ForceResolveName = gconf.ForceResolveName
               ,ForceResolveIp   = gconf.ForceResolveIp
          }
	local addr_t, err_msg = h.resolve_by_hosts(host.ip_src, nil, cfg)
	if not addr_t then
		if err_msg then	-- printing resolving error
			logger:warning("Hosts file was not processed: %s", err_msg)
		end
	else
		registrar = addr_t.domain
	end
	if not registrar then
		registrar, case, err_msg = h.get_registrar(host.ip_src, host.ip) 
	end
	logger:debug("Registrar address %q got from %s%s", registrar, case, err_msg and ". Errors: "..err_msg or "")

	local options = {
		 authUser = host.auth.user
		,authPass = host.auth.pass
		,timeout  = 2
	}

	if not feature or h.in_table (feature, {"FCF","BCF","CW"}) then
		local t = {["Login"] = ""}
		if args["username"] and args["username"]:find("unknown%d") then
			t["Login"] = "login"
		end

		t["Type"] = 0 -- disable forward
		t["CustomLogo"] = 0 -- disable logo
		t["BusyForward"] = ""
		t["AlwaysForward"] = ""
		t["CallWaiting"] = 1
		t["StType"] = ""

		-- Features
		if user_features["internal"] then
			-- BCF: do not apply BCF feature becase it sends "Moved temporarly"
			if user_features["internal"]["default"]["BCF"].status == "1" then
				t["Type"] = 2
				t["StType"] = 16
				t["BusyForward"] = user_features["internal"]["default"]["BCF"].value
			end

			-- FCF
			if user_features["internal"]["default"]["FCF"].status == "1" then
				t["Type"] = 1
				t["StType"] = 1
				t["AlwaysForward"] = user_features["internal"]["default"]["FCF"].value
			end
		end

		local eod = h.get_eod()
		if eod == "1" then
			eod = "2"
		elseif eod == "2" then
			eod = "1"
		end
		t["eod"] = eod

		-- Old version
		local str = "PAGEID=6&CONFIG_DATA=$AlwaysForward%26%2610%26$BusyForward%26%261%26$Type%260%26$Login%260%26%260%26%260%26%260%26%260%263000%2610%261%26%26%260%260%260%2610%2610%260%26%260%26603%26480"

		local mver, majver, uver, minver = host.fwversion:match("(%d+)%.(%d+)%.(%d+)%.(%d+)")
		local is_logo_used = h.is_exists("/var/www/images/logo.dob")
		if  tonumber (mver) == 2 and is_logo_used then
			t["CustomLogo"] = 2
		end

		if tonumber(majver) >= 61 then
			local phone_conf = {
				'$AlwaysForward',   -- Forward->Always->Target
				'',                 -- Forward->No Answer->Target
				'10',               -- Forward->No Answer->After Ring Time (values: 5, 10, 15, 20 seconds)
				'$BusyForward',     -- Forward->Busy->Target
				'',
				'$CallWaiting',     -- Call Waiting                        (0 - 'disabled', 1 - 'enabled')
				'$StType',
				'0',
				'$Login',           -- Hotline Number
				'0',
				'',
				'0',
				'',
				'0',
				'',
				'0',
				'',
				'0',
				'$eod',             -- Key As Send                          (0 - 'disabled', 1 - '#', 2 - '*')
				'',                 -- ReDialTone
				'',                 -- Emergency
				'0',                -- BusyToneDelay(seconds)               (values: 0, 3, 5)
				'0',                -- Ringer Device for Headset            (0 - 'Use Speaker', 1 - 'Use Headset')
				'0',                -- Auto redial                          (0 - 'disabled', 1 - 'enabled')
				'10',
				'10',
				'$CustomLogo',      -- Use Logo                             (0 - 'disabled', 1 - 'System Logo', 2 - 'Custom Logo')
				'',
				'0',
				'603',              -- Return code when refuse              (603 - 'Decline', 480 - 'Temporarily not available', 404 - 'Not found')
				'480',              -- Return code when DND                 (603 - 'Busy here', 480 - 'Temporarily not available', 404 - 'Not found')
				'',                 -- DND On Code
				'',                 -- DND Off Code
				'',                 -- Forward->Always->On Code
				'',                 -- Forward->Always->Off Code
				'',                 -- Forward->Busy->On Code
				'',                 -- Forward->Busy->Off Code
				'',                 -- Forvard->No Answer->On Code
				'',                 -- Forward->No Answer->Off Code
				'1',                -- Semi-Attend Transfer          	    (0 - 'disabled', 1 - 'enabled')
				'1',                -- Call Waiting Tone                    (0 - 'disabled', 1 - 'enabled')
				'1',
				'0',
				'1',                -- Intercom Tone                        (0 - 'disabled', 1 - 'enabled')
				'1',                -- Intercom Barge                        (0 - 'disabled', 1 - 'enabled')
				'0',                -- Call Completion                       (0 - 'disabled', 1 - 'enabled')
				'4',                -- Hotline delay                        (0 - 'disabled', 1 - 'enabled')  ?
				'1',                -- ??? Hotline Delay
				'1',                -- Attend Trans OnHook                  (0 - 'disabled', 1 - 'enabled')
				'1',                -- Feature Key Synchronisation          (0 - 'disabled', 1 - 'enabled')
				'1',                -- Time Out for Dial-now Rule
				'0',                -- Use Session Timer                   (0 - 'disabled', 1 - 'enabled')
				'7200',             -- Session Timer(seconds)(0~120s)
				h.get_button_sound(), -- Button Sound                         (0 - 'disabled', 1 - 'enabled')
				'1',                -- Send Sound                           (0 - 'disabled', 1 - 'enabled')
				'1',                -- Reserve # in User Name               (0 - 'disabled', 1 - 'enabled')
				'1',                -- Blind Transfer OnHook                (0 - 'disabled', 1 - 'enabled')
				'0',                -- Transfer on Conference Hang up       (0 - 'disabled', 1 - 'enabled')
				'0',                -- RFC 2543 Hold                        (0 - 'disabled', 1 - 'enabled')
				'1',                -- Use Outbound Proxy In Dialog         (0 - 'disabled', 1 - 'enabled')
				'1',                -- IsDeal180                            (0 - 'disabled', 1 - 'enabled')
				'0',                -- Logon Wizard                         (0 - 'disabled', 1 - 'enabled')
				'',                 -- PswLength
				'',                 -- PswPrefix
				'0',                -- PswDial                              (0 - 'disabled', 1 - 'enabled')
				'29',               -- Headset Send Volume (1~53)
				'',                 -- PushXML Server IP
				'0',
				'1',                -- Save Call History                    (0 - 'disabled', 1 - 'enabled')
				'0',                -- HideDTMFDelay                        (0 - 'disabled', 1 - 'enabled')
				'0',                -- Close Power Light                    (0 - 'disabled', 1 - 'enabled')
				'0',                -- HideDTMF                             (0 - 'disabled', 1 - 'enabled')
				'3',                -- DTMFRepetition                       (0, 1, 3)
				'',                 -- ActionURILimitIP
				'PCMA',             -- MulticastCodec                       (PCMU,PCMA,G729,G722,G726-16,G726-24,G726-32,G726-40,G723_53)
				'1',                -- PlayHoldTone
				'30',               -- PlayHoldToneDelay                    (0 - 'disabled', 1 - 'enabled')
				'',                 -- Tran Send DTMF
				'1',                -- AllowMute                            (0 - 'disabled', 1 - 'enabled')
				'0',                -- SRemoteName                          (0 - 'disabled', 1 - 'enabled')
				'3600',             -- SRemoteNameFlashTime(seconds)
				'0',                -- Dual Headset
				'0',                -- AutoAnswerDelay                      (0 - 'disabled', 1 - 'enabled')
				'0',                -- Headset Prior                        (0 - 'disabled', 1 - 'enabled')
				'1',                -- DsskeyBlindTran                      (0 - 'disabled', 1 - 'enabled')
				'0',                -- DTMFReplaceTran                      (0 - 'disabled', 1 - 'enabled')
				'0',                -- SendPoundKey                         (0 - 'disabled', 1 - 'enabled')
				'27',               -- Handset Send Volume (1~53)
				'39',               -- Handfree Send Volume (1~53)
				'1',                -- Fwd International                    (0 - 'disabled', 1 - 'enabled')
				'5',                -- ReLogOffTime(1~1000 minutes)
				'0',
				'1',                -- BLfNotifyType                        (0 - 'disabled', 1 - 'enabled')
				'0',                -- Show Transfer Select Menu            (0 - 'disabled', 1 - 'enabled')
				'0'                 -- Transfer Mode via DSSkey             (0 - 'Display number', 1 - 'Attended Transfer', 2 - 'Blind Transfer')
			}
			str = "PAGEID=6&CONFIG_DATA=" .. table.concat(phone_conf, "%26")
		elseif tonumber(majver) >= 60 then
			local phone_conf = {
				'$AlwaysForward',   -- Forward->Always->Target
				'',                 -- Forward->No Answer->Target
				'10',               -- Forward->No Answer->After Ring Time (values: 5, 10, 15, 20 seconds)
				'$BusyForward',     -- Forward->Busy->Target
				'',
				'$CallWaiting',     -- Call Waiting                        (0 - 'disabled', 1 - 'enabled')
				'$StType',
				'0',
				'$Login',           -- Hotline Number
				'0',
				'',
				'0',
				'',
				'0',
				'',
				'0',
				'',
				'0',
				'$eod',             -- Key As Send                          (0 - 'disabled', 1 - '#', 2 - '*')
				'',                 -- ReDialTone
				'',                 -- Emergency
				'0',                -- BusyToneDelay(seconds)               (values: 0, 3, 5)
				'0',                -- Ringer Device for Headset            (0 - 'Use Speaker', 1 - 'Use Headset')
				'0',                -- Auto redial                          (0 - 'disabled', 1 - 'enabled')
				'10',
				'10',
				'$CustomLogo',      -- Use Logo                             (0 - 'disabled', 1 - 'System Logo', 2 - 'Custom Logo')
				'',
				'0',
				'603',              -- Return code when refuse              (603 - 'Decline', 480 - 'Temporarily not available', 404 - 'Not found')
				'480',              -- Return code when DND                 (603 - 'Busy here', 480 - 'Temporarily not available', 404 - 'Not found')
				'',                 -- DND On Code
				'',                 -- DND Off Code
				'',                 -- Forward->Always->On Code
				'',                 -- Forward->Always->Off Code
				'',                 -- Forward->Busy->On Code
				'',                 -- Forward->Busy->Off Code
				'',                 -- Forvard->No Answer->On Code
				'',                 -- Forward->No Answer->Off Code
				'1',                -- Semi-Attend Transfer         	    (0 - 'disabled', 1 - 'enabled')
				'1',                -- Call Waiting Tone                    (0 - 'disabled', 1 - 'enabled')
				'1',
				'0',
				'1',                -- Intercom Mute                        (0 - 'disabled', 1 - 'enabled')
				'1',                -- Intercom Tone                        (0 - 'disabled', 1 - 'enabled')
				'0',                -- Intercom Barge                       (0 - 'disabled', 1 - 'enabled')
				'4',                -- Call Completion                      (0 - 'disabled', 1 - 'enabled')
				'1',                -- Hotline Delay
				'1',                -- Attend Trans OnHook                  (0 - 'disabled', 1 - 'enabled')
				'',                 -- Feature Key Synchronisation          (0 - 'disabled', 1 - 'enabled')
				'1',                -- Time Out for Dial-now Rule
				'0',                -- Use Session Timer                    (0 - 'disabled', 1 - 'enabled')
				'7200',             -- Session Timer(seconds)(0~120s)
				h.get_button_sound(),       -- Button Sound                         (0 - 'disabled', 1 - 'enabled')
				'1',                -- Send Sound                           (0 - 'disabled', 1 - 'enabled')
				'1',                -- Reserve # in User Name               (0 - 'disabled', 1 - 'enabled')
				'1',                -- Blind Transfer OnHook                (0 - 'disabled', 1 - 'enabled')
				'0',                -- Transfer on Conference Hang up       (0 - 'disabled', 1 - 'enabled')
				'0',                -- RFC 2543 Hold                        (0 - 'disabled', 1 - 'enabled')
				'1',                -- Use Outbound Proxy In Dialog         (0 - 'disabled', 1 - 'enabled')
				'1',                -- IsDeal180                            (0 - 'disabled', 1 - 'enabled')
				'0',                -- Logon Wizard                         (0 - 'disabled', 1 - 'enabled')
				'',                 -- PswLength
				'',                 -- PswPrefix
				'0',                -- PswDial                              (0 - 'disabled', 1 - 'enabled')
				'29',               -- Headset Send Volume (1~53)
				'',                 -- PushXML Server IP
				'0',
				'1',                -- Save Call History                    (0 - 'disabled', 1 - 'enabled')
				'0',                -- HideDTMFDelay                        (0 - 'disabled', 1 - 'enabled')
				'0',                -- Close Power Light                    (0 - 'disabled', 1 - 'enabled')
				'0',                -- HideDTMF                             (0 - 'disabled', 1 - 'enabled')
				'3',                -- DTMFRepetition                       (0, 1, 3)
				'0',                -- ActionURILimitIP
				'1'                 -- BLfNotifyType                        (0 - 'disabled', 1 - 'enabled')
			}
			str = "PAGEID=6&CONFIG_DATA=" .. table.concat(phone_conf, "%26")
		elseif host.model == "wp400" then
			str = "PAGEID=6&CONFIG_DATA=$AlwaysForward%26%2610%26$BusyForward%26%26$CallWaiting%26$StType%260%26$Login%260%26%260%26%260%26%260%26%260%261%26%26%260%260%260%2610%2610%260%26%260%26603%26480%26dnd*1*mute%26dnd*0*mute%26fcf*1*mute%26fcf*0*mute%26bcf*1*mute%26bcf*0*mute%26%26%261%261%261"
		end

		-- See order on the page (note order is important)
		local paramstring = string.gsub(str, "%$(%w+)", t)
		local answer = http:rawpost(host.ip, PORT_NUMBER, "/cgi-bin/ConfigManApp.com", paramstring, options) or {}
		logger:debug("Updating phone features for %s - %s", host.model, answer.code or 408)
	end
	
	if not feature then
		local person, tmp = {}
		local res = wmsldap:get("uid", tostring(uid))
		if #res ~= 0 then
			tmp, person = unpack (res[1])
		end
		if person["preferredLanguage"] == nil then
			person["preferredLanguage"] = h.get_admin_lang()
		end
		-- Get http proto
		local http_protocol = (host.model == "wp400" or gconf.HTTPForceUse) and "http" or "https"
		local http_port 	= (host.model == "wp400" or gconf.HTTPForceUse) and "80" or "443"
		
		local devFwInfo 	  = h.parse_firmware(host.fwversion)
		local model_spec	  = devFwInfo and devFwInfo["model"]
		local major_version	  = devFwInfo and devFwInfo["major"]
		local minor_version	  = devFwInfo and devFwInfo["minor"]
		local is_features_button = false
		if (host.model == "wp500" and (tonumber(minor_version) >= 13 or tonumber (major_version) >= 61) ) or ((host.model == "wp480" or host.model == "wp490") and (tonumber(minor_version) >= 10 or tonumber (major_version) >= 61)) then
			is_features_button = true
		end

		-- XML browser (user features)
		if is_features_button then
			local url = string.gsub(
				"$protocol://$registrar:$port/?action=getxmlbrowser%26get=features", "%$(%w+)",
				{
					protocol  = http_protocol,
					registrar = registrar,
					port      = http_port,
				}
			)
			local tr_Features = "Featur."
			if person["preferredLanguage"] == "it" then
				tr_Features = "Funz."
			elseif person["preferredLanguage"] == "fr" then
				tr_Features = "Fcts."
			elseif person["preferredLanguage"] == "de" then
				tr_Features = "Einstel."
			elseif person["preferredLanguage"] == "ru" then
				tr_Features = "Функц"
			end
			local s = table.concat({
				"2&27",
				"0",
				tr_Features,
				url,
				"exp"
			}, "$")
			local paramstring = "PAGEID=35&CONFIG_DATA=" .. http.urlEncode(s)
			local answer = http:rawpost(host.ip, PORT_NUMBER, "/cgi-bin/ConfigManApp.com", paramstring, options) or {}
			logger:debug("Updating DSS programable keys for %s - %s", host.model, answer.code or 408)
		end

		-- Phonebooks
		do
			local pbooknames = "searchall,Directory," .. table.concat(core.get_list_phonebook(uid), ",")
			local t = {
				["pbookname1"] = "", ["link1"] = "",
				["pbookname2"] = "", ["link2"] = "",
				["pbookname3"] = "", ["link3"] = "",
				["pbookname4"] = "", ["link4"] = "",
				["pbookname5"] = "", ["link5"] = "",
			}
			if pbooknames ~= "" then
				local pbookst = h.split(pbooknames, ",")
				local pbookst_names = core.get_phonebook_names(pbookst)
				for i, name in ipairs(pbookst) do
					local pbookname = (i == 1 and "searchall" or (pbookst_names[name] or ""))
					pbookname = (i == 2 and "Directory" or (pbookst_names[name] or ""))
					local link = string.gsub(
						"$protocol://$registrar:$port/?action=getphonebook&phonebook=$phonebook&input=#SEARCH", "%$(%w+)",
						{
							protocol  = http_protocol,
							registrar = registrar,
							port      = http_port,
							phonebook = http.urlEncode(name:lower())
						}
					)
	
					if pbookname == "Directory" then 
						if person["preferredLanguage"] == "it" then
							pbookname = "Rubrica Interni"
						elseif person["preferredLanguage"] == "fr" then
							pbookname = "Annuaire"
						elseif person["preferredLanguage"] == "de" then
							pbookname = "Ordner"
						elseif person["preferredLanguage"] == "ru" then
							pbookname = "Контакты"
						end
					end
					if pbookname == "searchall" then
						if person["preferredLanguage"] == "it" then
							pbookname = "Tutti"
						elseif person["preferredLanguage"] == "fr" then
							pbookname = "Tous"
						elseif person["preferredLanguage"] == "de" then
							pbookname = "Alle"
						elseif person["preferredLanguage"] == "ru" then
							pbookname = "Все"
						else
							pbookname = "All"
						end
					end
	
					t["link" .. i] = http.urlEncode(link)
					t["pbookname" .. i] = http.urlEncode(pbookname)
				end
	
				local str = "PAGEID=30&CONFIG_DATA=$link1%C3%BE$link2%C3%BE$link3%C3%BE$link4%C3%BE$link5%C3%BE$pbookname1%C3%BE$pbookname2%C3%BE$pbookname3%C3%BE$pbookname4%C3%BE$pbookname5"
	
				-- See order on the page (note order is important)
				local paramstring = string.gsub(str, "%$(%w+)",t)
				local answer = http:rawpost(host.ip, PORT_NUMBER, "/cgi-bin/ConfigManApp.com", paramstring, options) or {}
				logger:debug("Updating remote phonebooks for %s - %s", host.model, answer.code or 408)
			end
		end
	end
end
