module("resources.wildixfxo", package.seeall)


-- {{ Codecs order for narrowband & wideband
--------------------------------------------------
-- Audio codec priority (0: PCMU, 4: G723, 8:PCMA, 18:G729A, -1: none)
-- l1ac1=18
-- l1ac2=4
-- l1ac3=8
-- l1ac4=0
-- l1ac5=-1

--------------------------------------------------
codecs = {
    ["wideband"] = {
        ["l1ac1"] = "8",  -- PCMA
        ["l1ac2"] = "0",  -- PCMU
        ["l1ac3"] = "18",  -- G729
    },
    ["narrowband"] = {
        ["l1ac1"] = "18",  -- G729
        ["l1ac2"] = "8",  -- PCMA	
        ["l1ac3"] = "0",  -- PCMU
    }
}

-- default settings per port
port_default = {
   ["port_enabled"]									= "0", -- Enable Port
   ["port_hotline_number"]							= "", -- Hotline numbers - fxo1.hotlinetel
   ["port_flash_time"]								= "300", -- Flash time (msec) - fxo1.FlashTime
   ["port_cid_type"]								= "1", -- Bellcore FSK, ETSI FSK - fxo1.calleridtype
   ["port_preversal_detection"]						= "0", -- Polarity reversal detection - fxo1.polarityreversaldetection
   ["port_answer_time"]								= "1", -- Dialing answer delay time - fxo1.dialanswertime
   ["port_pstn_ring_count"]							= "1", -- PSTN ring count - fxo1.answerringcnt
   ["port_fax_type"]								= "2", -- FAX type disabled, bypass, T38 - fxo1.faxrelay
   ["port_codec_input_gain"] 						= "32", -- Input Gain - fxo1.voicevolume=32 (22 .. 37)
   ["port_codec_output_gain"] 						= "32", -- Output Gain - fxo1.remotercvvolume=32 (22 .. 37)
}

-- models & port count
models = {
    ["w04fxo"] = 4,
    --["w04fxs"] = 4
}

parameters = {
   ["Firmware"] = {
   	  -- ["frmurl"] = "/firmwares/fxofxs/fxo/",
   	  -- ["fm_updatetime"] = "99:99"
   },

   ["Devconf"] = {
	  ["pabxmode"] = "0",

	  -- Network
	  -- ["dhcp"] = "0", -- 0 -static, 1 - dynamic (default: 1)
	  ["dt60"] = "", -- dhcpd options 60 and 61
	  ["dt61"] = "",
	  -- ["vlan"] = "0", -- vlan: enable/disable (if checkbox disable not send nothing)
	  -- ["ipa"] = "192.168.222.99", -- ipaddr
	  -- ["ipm"] = "255.255.255.0", -- netmask
	  -- ["ipgw"] = "192.168.222.254", -- gateway
	  ["pppoeu"] = "",
	  ["pppoep"] = "",
	  -- ["dns1"] = "217.199.1.1",
	  -- ["dns2"] = "217.199.0.250",
	  ["tldp"] = "23", -- telnet port
	  ["httpp"] = "80",
	  ["httpsp"] = "443",
	  -- ["ntps"] = "pool.ntp.org", -- ntp server
	  ["ntpt"] = "3600", -- ntp refresh interval
	  ["tz"] = "2.0", -- timezone (interval -13.5h  +13.5h, default 0)

	  ["dlt"] = "0", -- Daylight saving 
	  ["dltb"] = "0,0,0,0", -- Standard saving start time (month (1-12), week (1-5), week days (1-7), hour (0-24))
	  ["dltbs"] = "1", -- Daylight saving bias +/- hours (1, 1.5 or 2)
	  ["dlte"] = "0,0,0,0", -- Standard  time start time (month (1-12), week (1-5), week days (1-7), hour (0-24))

	  -- Users access
	  ["u1"] = "root",
--	  ["u1p"] = "wil!superroot!dix",
	  ["u1langid"] = "0",
	  ["u2"] = "admin",
--	  ["u2p"] = "wildix",
	  ["u2langid"] = "0",
	  ["u3"] = "user",
--	  ["u3p"] = "wildix",
	  ["u3langid"] = "0",

	  -- Provisioning
	  ["provtype"] = "3",
	  ["ems"] = "0",
	  ["emsip"] = "",
	  ["emsp"] = "61003",
	  ["emsdp"] = "61005",
	  ["httpurl"] = "http://192.168.222.25/firmwares/welltech/2540/",
	  ["emssri"] = "3600",
	  ["httpcri"] = "14400",
	  ["httpcu"] = "admin",
	  ["httpcp"] = "",
	  
	  -- Template for siptk
--	  ["siptkX"] = { -- siptk1 .. siptk4
--		 ["regtype"] = "0", -- register:0, predefine:1, default is 1
--		 ["domain"] = "192.168.222.25",
--		 ["proxy"] = "192.168.222.25",
--		 ["proxyport"] = "5060",
--		 ["outboundproxy"] = "192.168.222.25",
--		 ["outboundproxyport"] = "5060",
--		 ["ttl"] = "300", -- Register time to live (sec)
--		 ["telno"] = "fxo0_0", -- fxo0_0 .. fxo3_0
--		 ["userid"] = "fxo0_0",
--		 ["password"] = "1000AAAA",
--		 ["displayname"] = "FXO line1",
--		 ["rejectanonymcall"] = "0",
--		 ["outcallerid_displayname"] = "1", -- none:0, PSTN caller ID:1, SIP user ID:2, FXO TEL no:3, default is 1
--		 ["outcallerid_userid"] = "1", -- SIP user ID:0, PSTN caller ID:1, FXO TEL no:2, default is 1
--		 ["twostagedial"] = "0",
--	  },
	  
	  ["line1"] = { -- line1 .. line4
		 ["active"] = "1"
	  },

	  ["fxo1"] = { -- fxo1 .. fxo4
		 ["telno"] = "2001",
		 ["polarityreversaldetection"] = "0",
		 ["curdropfordisconn"] = "1",
		 ["curdropfordisconn"] = "1",
		 ["incomingcallhandling"] = "0", -- Incoming calls handling (hotline:0, 2 stage dialing:1, default 1)
		 ["hotlinetel"] = "2001", -- Hotline numbers
		 ["faxrelay"] = "2", -- disabled:0, bypass:1, T38:2, default 2
		 ["voicevolume"] = "32", -- Input (encode) Gain  (interval 22~37 (-10db ~ +5db), default 32(0db)
		 ["remotercvvolume"] = "32", -- Output (decode) Gain (interval 22~37 (-10 ~ +5db), default 32(0db))
		 ["dialanswertime"] = "0",
		 ["answerringcnt"] = "0",
		 ["calleridtype"] = "1", -- none:-1, Bellcore FSK:0, ETSI FSK:1, ETSI DTMF:3, Danish DTMF:4, Indian DTMF:5, Brazilian DTMF:6, default -1
	  },

	  ["eod"] = "0", -- End of Digit char, 0 -> none, 1 -> *, 2 -> #
	  ["stm"] = "0", -- SIP session timer (disabled:0, enabled:1)
	  ["stme"] = "", -- Session timer expires (seconds)
	  ["stmse"] = "",
	  ["prack"] = "0",
	  ["slpt"] = "8080",
	  ["sqos"] = "46", -- DSCP Value (0~63)
	  ["sqost"] = "1", -- SIP QoS Type (1 - DiffServ)
	  ["pxonly"] = "1",	-- Accept SIP mesages Proxy Only (0 - disable, 1 - enable)
	  ["l1hldt"] = "0",
	  ["l1scph"] = "0",
	  ["cvid"] = "0",
	  ["stmr"] = "UAC",
	  ["sipinfoflash"] = "1",
	  ["slog"] = "0",	 		-- Syslog (0 - disable, 1 - enable)
	  ["slogp"] = "514",		-- Syslog port
	  ["slogs"]	= "127.0.0.1",	-- Syslog server
	  ["slogb"] = "000000000000", 			-- Syslog Start (enable Check for start from Any Time)
	  ["sloge"] = "999999999999", 			-- Syslog Stop (enable Check for start from Any Time)
	  ["dbl"] = "phoneMgr=0,SipMgr=0,sntp=0,devmgr=0,emsclt=0", 	-- Debug level for enabled modules
	  ["dbm"] = "phoneMgr=0,SipMgr=0,sntp=0,devmgr=0,emsclt=0",		-- Debug modules
	  ["DSPDebug"] = "0",				-- DSP Debug (0 - disable, 1 - enable)
	  ["DSPLogPort"] = "50000",			-- DSP Capture Port
	  ["DSPLogServer"] = "127.0.0.1",	-- DSP Capture Server
	  ["slogen"] = "0",				-- Event notice syslog (0 - disable, 1 - enable)
	  ["enslogp"] = "514",			-- Event notice syslog port
	  ["enslogs"] = "127.0.0.1",		-- Event notice syslog server
	  ["l1ac1"] = "-1",
	  ["l1ac2"] = "-1",
	  ["l1ac3"] = "-1",
	  ["l1ac4"] = "-1",
	  ["l1ac5"] = "-1",
	  ["l1711up"] = "20",
	  ["l1723p"] = "30",
	  ["l1723br"] = "1",
	  ["l1711ap"] = "20",
	  ["l1729p"] = "20",	-- ptime for g729
	  ["l1acpr"] = "0",
	  ["silc"] = "0",
	  ["tSilc"] = "60",
	  ["rtpb"] = "16384",
	  ["rqos"] = "46", -- DSCP Value (0~63)
	  ["rqost"] = "1", -- RTP QoS Type (1 - DiffServ)
	  ["l12833p"] = "101",
	  ["agcm"] = "0",
	  ["ijitb"] = "60",
	  ["xjitb"] = "150",	-- Dynamic Jitter Buffer Optimization Factor
	  ["jitof"] = "7",

	  ["siprsp_noanyline"] = "503", -- Line Congestion Code
	  ["l1dtmfr"] = "2", -- SIP INFO
	  ["dtmfsn"] = "70", -- DTMF Send On Time (msec)
	  ["dtmfsf"] = "70", -- DTMF Send Off Time (msec)
	  ["dtmfdmn"] = "60", -- DTMF Detect Min On Time (msec)
	  ["dtmfdmf"] = "60", -- DTMF Detect Min Off Time (msec) 
	  ["dtmfrv"] = "0", -- DTMF Relay Volume (interval is 0db ~ -31db, values are 0 ~ 32, default is 0)
	  
	  ["t38v"] = "12", -- T38 Volume (interval is -21db ~ -6db, values are 0 ~ 15, default is 12)
	  ["xecl"] = "64", -- Max Echo Tail Length(G.168)
	  ["tbr21"]= "0",  -- DC Impedance. 0 - disabled, 1 - enabled.
	  
	  
	  -- ["route1"] = { -- route1 .. route8
	  -- 	 ["enable"] = "1",
	  -- 	 ["incomingcalltype"] = "1", -- 0 -voip, 1 - fxo
	  -- 	 ["matchprefix"] = "2001", -- 2001, 2002 ... 
	  -- 	 ["matchedincominglist"] = "1", -- line1:1, line2:2, line3:4, line4:8, all lines:15
	  -- 	 ["matchlen"] = "4",
	  -- 	 ["noanstimeout"] = "120",
	  -- 	 ["primary"] = {
	  -- 		["outgoingtype"] = "0", -- 0 -voip, 1 - fxo
	  -- 		["huntingtype"] = "1", -- Priority Ring:0,Cyclic Ring:1,Simultaneous Ring:2
	  -- 		["routinglist"] = "0", -- none:-1, trunk1: 0, trunk2: 1, trunk3: 2, trunk4: 3
	  -- 		["huntingcycle"] = "1",
	  -- 	 },
	  -- 	 ["priext"] = {
	  -- 		["dmgroup"] = "0",
	  -- 	 },
	  -- 	 ["bakext"] = {
	  -- 		["bakrouteactive"] = "0", -- disabled:0, enabled:1
	  -- 		["reroutedmgroup"] = "0",
	  -- 	 },
	  -- 	 ["bak"] = {
	  -- 		["outgoingtype"] = "",
	  -- 		["huntingtype"] = "",
	  -- 		["routinglist"] = "",
	  -- 		["huntingcycle"] = "",			
	  -- 	 }
	  -- }
   }
}
