module("resources.wildixfxs4", package.seeall)

-- {{ Codecs order for narrowband & wideband
--------------------------------------------------
-- Audio codec priority (0: PCMU, 4: G723, 8:PCMA, 18:G729A, -1: none)
-- l1ac1=18
-- l1ac2=4
-- l1ac3=8
-- l1ac4=0
-- l1ac5=-1

--------------------------------------------------
codecs = {
    ["wideband"] = {
        ["l1ac1"] = "8",  -- PCMA
        ["l1ac2"] = "0",  -- PCMU
        ["l1ac3"] = "18",  -- G729
    },
    ["narrowband"] = {
        ["l1ac1"] = "18",  -- G729
        ["l1ac2"] = "8",  -- PCMA	
        ["l1ac3"] = "0",  -- PCMU
    }
}

-- models & port count
models = {
    ["w04fxs"]	= 4
}

parameters = {
    ["Firmware"] = {
        --["frmurl"] = "/firmwares/fxofxs/fxs/",
        --["fm_updatetime"] = "99:99"
    },

    ["Devconf"] = {
        ["pabxmode"] = "0",

        -- Network
        -- ["dhcp"] = "0", -- 0 -static, 1 - dynamic (default: 1)
        ["dt60"] = "", -- dhcpd options 60 and 61
        ["dt61"] = "",
        -- ["vlan"] = "0", -- vlan: enable/disable (if checkbox disable not send nothing)
        -- ["ipa"] = "192.168.222.99", -- ipaddr
        -- ["ipm"] = "255.255.255.0", -- netmask
        -- ["ipgw"] = "192.168.222.254", -- gateway
        ["pppoeu"] = "",
        ["pppoep"] = "",
        -- ["dns1"] = "217.199.1.1",
        -- ["dns2"] = "217.199.0.250",
        ["tldp"] = "23", -- telnet port
        ["httpp"] = "80",
        ["httpsp"] = "443",
        -- ["ntps"] = "pool.ntp.org", -- ntp server
        ["ntpt"] = "3600", -- ntp refresh interval
        ["tz"] = "2.0", -- timezone (interval -13.5h  +13.5h, default 0)

        ["dlt"] = "0", -- Daylight saving 
        ["dltb"] = "0,0,0,0", -- Standard saving start time (month (1-12), week (1-5), week days (1-7), hour (0-24))
        ["dltbs"] = "1", -- Daylight saving bias +/- hours (1, 1.5 or 2)
        ["dlte"] = "0,0,0,0", -- Standard  time start time (month (1-12), week (1-5), week days (1-7), hour (0-24))

        -- SIP proxy params
        ["px"] = {
            ["registrardomain"] = "",
            ["primary"] = {
                ["registrarpx"] = "", -- proxy server
                ["registrarport"] = "5060", -- proxy server port
                ["outxpx"] = "", -- Outbound sever proxy
                ["outxpxport"] = "5060" -- outbound proxy sever port
            },
            ["havesecondpx"] = "0", -- secondary proxy server status (disabled:0, enabled:1, default 0)
            ["second"] = {
                ["registrarpx"] = "", -- proxy server
                ["registrarport"] = "5060", -- proxy server port
                ["outxpx"] = "", -- Outbound sever proxy
                ["outxpxport"] = "5060" -- outbound proxy sever port
            },
            ["registerexpires"] = "120" -- SIP Register Expires (seconds) (default 120sec)
        },

        -- Users access
        ["u1"] = "root",
--        ["u1p"] = "wil!superroot!dix",
        ["u1langid"] = "0",
        ["u2"] = "admin",
--        ["u2p"] = "wildix",
        ["u2langid"] = "0",
        ["u3"] = "user",
--        ["u3p"] = "wildix",
        ["u3langid"] = "0",

        -- Caller ID
        ["fxs_caller_id"] = {
            ["calleridtype"] = "1", -- Caller ID Mode (0: FSK Bellcore, 1: FSK ETSI , 2: DTMF)
            ["reversepolaritybefcallerid"] = "0", -- Polarity Reverse Before Caller ID (1: Yes,  0:No)
            ["dualtonebefcallerid"] = "1", -- Dual Tone Before Caller ID (1: Yes, 0:No)
            ["calleridpresent"] = "0" -- Caller ID Present (0:Before First Ring , 1:After First Ring ,  2:After First Short Ring)
        },

        -- Provisioning
        ["provtype"] = "3",
        ["ems"] = "0",
        ["emsip"] = "",
        ["emsp"] = "61003",
        ["emsdp"] = "61005",
        ["httpurl"] = "http://192.168.222.25/firmwares/welltech/2540/",
        ["emssri"] = "3600",
        ["httpcri"] = "14400",
        ["httpcu"] = "admin",
        ["httpcp"] = "",

        ["line1"] = { -- line1 .. line4
            ["active"] = "1"
        },

        ["fxs1"] = { -- fxs1 .. fxS4
            ["reg"] = "0", -- Register type (register:0, predefine:1, internal:2, default 1)
            ["telno"] = "1001",
            ["userid"] = "fxs0_0",
            ["password"] = "1000AAAA",
            ["displayname"] = "FXS line1",
            ["forwardreason"] = "0", -- Forward type (null:0, unconditional:1, busy:2, no answer:4, busy+no answer:6, default is 0)
            ["forwardtel"] = "",
            ["noanstimeout"] = "120", -- No answer time out (seconds) (default 120 sec)
            ["callwaiting"] = "3", -- Call waiting (enabled:3, disabled:2, default 3)
            ["rejectanonymcall"] = "0", -- Reject anonymous calls (disabled:0, enabled:1, default 0)
            ["hotline"] = "0", -- Hotline type(disabled:0, enabled:1, default 0)
            ["hotlinetel"] = "", -- Hotline number
            ["polarityreversal"] = "0", -- Polarity reversal generation (disabled:0, enabled:1, default is 0)
            ["curdropfordisconn"] = "1", -- Current drop generation (disabled:0, enabled:1,default is 1)
            ["voicevolume"] = "32", -- Input (encode) Gain  (interval 22~37 (-10db ~ +5db), default 32(0db)
            ["remotercvvolume"] = "32", -- Output (decode) Gain (interval 22~37 (-10 ~ +5db), default 32(0db))
            ["faxrelay"] = "2", -- disabled:0, bypass:1, T38:2, default 2
            ["vms"] = "0", -- Voice Mail subscription (disabled:0, enabled:1, default is 0)
            ["calleridtype"] = "1", -- Caller ID (inhibit:0, transparent:1, default 0)
            ["calleridtovoip"] = "1" -- Caller ID SIP (inhibit:0, transparent:1, default 0)
        },

        ["eod"] = "0", -- End of Digit char, 0 -> none, 1 -> *, 2 -> #
        ["stm"] = "0", -- SIP session timer (disabled:0, enabled:1)
        ["stme"] = "", -- Session timer expires (seconds)
        ["stmse"] = "",
        ["prack"] = "0",
        ["slpt"] = "8080",
        ["sqos"] = "46", -- DSCP Value (0~63)
        ["sqost"] = "1", -- SIP QoS Type (1 - DiffServ)
        ["pxonly"] = "1",	-- Accept SIP mesages Proxy Only (0 - disable, 1 - enable)
        ["l1hldt"] = "0",
        ["l1scph"] = "0",
        ["cvid"] = "0",
        ["stmr"] = "UAC",
        ["sipinfoflash"] = "0",		-- SIP-Info Flash Mode
        ["slog"] = "0",	 			-- Syslog (0 - disable, 1 - enable)
	  	["slogp"] = "514",			-- Syslog port
	  	["slogs"] = "127.0.0.1",	-- Syslog server
	  	["slogb"] = "000000000000", 			-- Syslog Start (enable Check for start from Any Time)
	  	["sloge"] = "999999999999", 			-- Syslog Stop (enable Check for start from Any Time)
--	    ["dbl"] = "phoneMgr=7,SipMgr=7,sntp=7,devmgr=7,emsclt=7", 		-- ??
--	    ["dbm"] = "phoneMgr=15,SipMgr=15,sntp=15,devmgr=15,emsclt=15",	-- ??
	  	["DSPDebug"] = "0",				-- DSP Debug (0 - disable, 1 - enable)
		["DSPLogPort"] = "50000",		-- DSP Capture Port
		["DSPLogServer"] = "127.0.0.1",	-- DSP Capture Server
	  	["slogen"] = "0",				-- Event notice syslog (0 - disable, 1 - enable)
	 	["enslogp"] = "514",			-- Event notice syslog port
	  	["enslogs"] = "127.0.0.1",		-- Event notice syslog server
        ["l1ac1"] = "-1",
        ["l1ac2"] = "-1",
        ["l1ac3"] = "-1",
        ["l1ac4"] = "-1",
        ["l1ac5"] = "-1",
        ["l1711up"] = "20",
        ["l1723p"] = "30",
        ["l1723br"] = "1",
        ["l1711ap"] = "20",
        ["l1729p"] = "20",	-- ptime for g729
        ["l1acpr"] = "0",
        ["l1dtmfr"] = "2", -- SIP INFO
        ["silc"] = "0",
        ["rtpb"] = "16384",
        ["rqos"] = "46", -- DSCP Value (0~63)
        ["rqost"] = "1", -- RTP QoS Type (1 - DiffServ)
        ["l12833p"] = "101",
        ["agcm"] = "0",
        ["ijitb"] = "60",
        ["xjitb"] = "150",	-- Dynamic Jitter Buffer Optimization Factor
        ["jitof"] = "7",

        ["fxsMinFla"] = "8", -- Minimum Flash Time (unit: 10msec) (interval 80-800ms)
        ["fxsMaxFla"] = "30", -- Max Flash Time (unit: 10msec) (interval 80-800ms)
        ["curdrpms"] = "300", -- Current drop time (msec)
        
        ["xecl"] = "64", -- Max Echo Tail Length(G.168)
		
    	["sring"]= "0", -- Short ring detection
		
        -- RingTone cadence (ms)
        ["t1ron"] = "1000",     --internal
        ["t1roff"] = "2000",    
        ["t2ron"] = "2100",     --external
        ["t2roff"] = "3400",
        ["t3ron"] = "800",      --blacklist
        ["t3roff"] = "3200",
        ["t4ron"] = "1600",     --whitelist
        ["t4roff"] = "1300",
        ["t5ron"] = "500",      --silent
        ["t5roff"] = "8000",
        ["t6ron"] = "1200",     --custom
        ["t6roff"] = "5500",


	  -- ["route1"] = { -- route1 .. route8
	  -- 	 ["enable"] = "1",
	  -- 	 ["incomingcalltype"] = "1", -- 0 -voip, 1 - fxo
	  -- 	 ["matchprefix"] = "2001", -- 2001, 2002 ... 
	  -- 	 ["matchedincominglist"] = "1", -- line1:1, line2:2, line3:4, line4:8, all lines:15
	  -- 	 ["matchlen"] = "4",
	  -- 	 ["noanstimeout"] = "120",
	  -- 	 ["primary"] = {
	  -- 		["outgoingtype"] = "0", -- 0 -voip, 1 - fxo
	  -- 		["huntingtype"] = "1", -- Priority Ring:0,Cyclic Ring:1,Simultaneous Ring:2
	  -- 		["routinglist"] = "0", -- none:-1, trunk1: 0, trunk2: 1, trunk3: 2, trunk4: 3
	  -- 		["huntingcycle"] = "1",
	  -- 	 },
	  -- 	 ["priext"] = {
	  -- 		["dmgroup"] = "0",
	  -- 	 },
	  -- 	 ["bakext"] = {
	  -- 		["bakrouteactive"] = "0", -- disabled:0, enabled:1
	  -- 		["reroutedmgroup"] = "0",
	  -- 	 },
	  -- 	 ["bak"] = {
	  -- 		["outgoingtype"] = "",
	  -- 		["huntingtype"] = "",
	  -- 		["routinglist"] = "",
	  -- 		["huntingcycle"] = "",			
	  -- 	 }
	  -- }
    }
}