--- Static provision data for new Wp480 phone
-- 
-- @file	wildixp2.lua 
-- @author	Vyacheslav Khudan
-- @created 2014/10/30

module("resources.wildixp2", package.seeall)

--- @note: Codecs priority has back order !!!
codecs = {	-- account audio codecs settings (NB: priority has back order: 5 - 4 - 3 - 2 - 1)
	["wideband"]   = {	-- opus - PCMA - PCMU - G729
		["Audio0"] = {
			["Enable"]   = "1",
			["Name"]     = "PCMU",
			["Priority"] = "3",
			["Payload"]  = "0",
		},
		["Audio1"] = {
			["Enable"]   = "1",
			["Name"]     = "PCMA",
			["Priority"] = "4",
			["Payload"]  = "8",
		},
		["Audio2"] = {
			["Enable"]   = "0",
			["Name"]     = "G723_53",
			["Priority"] = "0",
			["Payload"]  = "4",
		},
		["Audio3"] = {
			["Enable"]   = "0",
			["Name"]     = "G723_63",
			["Priority"] = "0",
			["Payload"]  = "4",
		},
		["Audio4"] = {
			["Enable"]   = "1",
			["Name"]     = "G729",
			["Priority"] = "1",
			["Payload"]  = "18",
		},
		["Audio5"] = {
			["Enable"]   = "0",
			["Name"]     = "G722",
			["Priority"] = "5",
			["Payload"]  = "9",
		},
		["Audio6"] = {
			["Enable"]   = "0",
			["Name"]     = "G726-16",
			["Priority"] = "2",
			["Payload"]  = "112",
		},
		["Audio7"] = {
			["Enable"]   = "0",
			["Name"]     = "G726-24",
			["Priority"] = "0",
			["Payload"]  = "102",
		},
		["Audio8"] = {
			["Enable"]   = "0",
			["Name"]     = "G726-32",
			["Priority"] = "0",
			["Payload"]  = "2",
		},
		["Audio9"] = {
			["Enable"]   = "0",
			["Name"]     = "G726-40",
			["Priority"] = "0",
			["Payload"]  = "104",
		},
		["Audio10"] = {
			["Enable"]   = "0",
			["Name"]     = "iLBC_13_3",
			["Priority"] = "0",
			["Payload"]  = "97",
		},
		["Audio11"] = {
			["Enable"]   = "0",
			["Name"]     = "iLBC_15_2",
			["Priority"] = "0",
			["Payload"]  = "97",
		},
	},
	
	["narrowband"] = { -- opus - G729 - PCMA - PCMU
		["Audio0"] = {
			["Enable"]   = "1",
			["Name"]     = "PCMU",
			["Priority"] = "2",
			["Payload"]  = "0",
		},
		["Audio1"] = {
			["Enable"]   = "1",
			["Name"]     = "PCMA",
			["Priority"] = "3",
			["Payload"]  = "8",
		},
		["Audio2"] = {
			["Enable"]   = "0",
			["Name"]     = "G723_53",
			["Priority"] = "0",
			["Payload"]  = "4",
		},
		["Audio3"] = {
			["Enable"]   = "0",
			["Name"]     = "G723_63",
			["Priority"] = "0",
			["Payload"]  = "4",
		},
		["Audio4"] = {
			["Enable"]   = "1",
			["Name"]     = "G729",
			["Priority"] = "5",
			["Payload"]  = "18",
		},
		["Audio5"] = {
			["Enable"]   = "0",
			["Name"]     = "G722",
			["Priority"] = "1",
			["Payload"]  = "9",
		},
		["Audio6"] = {
			["Enable"]   = "0",
			["Name"]     = "G726-16",
			["Priority"] = "4",
			["Payload"]  = "112",
		},
		["Audio7"] = {
			["Enable"]   = "0",
			["Name"]     = "G726-24",
			["Priority"] = "0",
			["Payload"]  = "102",
		},
		["Audio8"] = {
			["Enable"]   = "0",
			["Name"]     = "G726-32",
			["Priority"] = "0",
			["Payload"]  = "2",
		},
		["Audio9"] = {
			["Enable"]   = "0",
			["Name"]     = "G726-40",
			["Priority"] = "0",
			["Payload"]  = "104",
		},
		["Audio10"] = {
			["Enable"]   = "0",
			["Name"]     = "iLBC_13_3",
			["Priority"] = "0",
			["Payload"]  = "97",
		},
		["Audio11"] = {
			["Enable"]   = "0",
			["Name"]     = "iLBC_15_2",
			["Priority"] = "0",
			["Payload"]  = "97",
		}
	}
}

paramaters_init = {	-- init configuration (used in case phone have to upgrade)
	["Autoprovision"] = {		  			--- Auto-Provision Configuation
		["GENERAL"] 		= {					-- access to the auto provision server
			["Url"] 	  	 		= "",
			["Username"]  	 		= "",
			["Pwd"] 	  	 		= "",
		},
		["MODE"]	     	= { 				-- autoprovision mode
			["Mode"] 	  	 		= "1",			-- 0:disable(default); 1:Power On; 2:Repeatedly; 3:Power On + Repeatedly
		},	 
		["SCHEDULE"]	 	= { 				-- the schedule time for auto provision when the auto provision mode is Repeatedly or Power On + Repeatedly
			["DayOfWeek"] 	 		= "", 			-- 0 is for everyday, 1~7 is for monday~sunday
			["HourOfDay"] 	 		= "", 			-- 0~23
		},
		["AES"]				= {					-- AES key for decryptin configure files
			["Key16"]     	 		= "",			-- the common config file
			["Key16Mac"]  	 		= "",			-- the Mac-Oriented config file
		},
		["PNP"]				= {					-- PnP feature
			["Enable"]    	 		= "0",			-- 0:Disabled; 1:Enabled(default)
		},
		["DHCP_OPTION"] 	= {					-- the DHCP custom option
			["CustomId"] 	 		= "128",
		},
		["RETRY_TIMES"]		= {					-- Autoprovision attempts
			["Times"]				= "1",
		},
		["Reboot"]			= {
			["Enable"]    	 		= "0",
		}
	},
}

parameters = { -- basic configuration
	["Settings"] 	  = {			  		--- Phone Settings Configuation 
	  	["GENERAL"]			= {					-- some general configuration
	  		["DirectIP"]			= "0"		-- enable/disable direct IP calls
	  	},
		["LOGLEVEL"]   		= {					-- the log level settings
			["Level"]        		= "7",			-- values: 0, 1, 2, 3(default), 4, 5, 6, 7
			["RemoteSyslog"] 		= "0",			-- enable/disable remote syslog server
			["RemoteServer"] 		= "",			-- syslog server address
		},
		["HANDFREE"]		= {					-- hand free volume configuration
			["SpkMidValue"]			= "18",			-- speaker gain level
			["MicMidValue"]			= "18",			-- microphone gain level
			["SigToneMidVol"]		= "18",			-- tone signal gain level
		},
		["HEADSET"]			= {					-- head set volume configuration
			["Active"]				= "0",			-- enable/disable headset mode
			["SigToneMidVol"]		= "14",			-- tone signal gain level
			["SpkMidValue"]			= "16",			-- microphone gain level
			["MicMidValue"]			= "16",			-- speaker gain level
			["PlayRing"]			= "0",			-- headset play ringtone: 0 - no, 1 - yes.
		},
		["HANDSET"]			= {					-- hand set volume configuration
			["SigToneMidVol"]		= "20",			-- tone signal gain level
			["SpkMidValue"]			= "20",			-- speaker gain level
			["MicMidValue"]			= "20",			-- microphone gain level
		},
		["WEB_LOGIN"]	  	= {					-- the login password of the web
			["Active"]		 		= "1",			-- disable/enable setting via WEB interface
			["User"]		 		= "admin",
			["Password"]	 		= "admin",		-- admin by default
			["Type"]				= "1",		-- type of user
			["User02"]				= "user",		
			["Password02"]			= "user",
			["Type02"]				= "0",
		},
		["LANGUAGE"]	   	= {					-- language settings (0 - English, 3 - Russian, 6 - Spanish, 7 - Italian, 9 - French, 10 - Deutsch)
			["Type"]		 		= "0",			-- LCD language
			["ActiveWebLang"]		= "1",			-- activate using 'WebLang' parameter for WEB language, otherwise will be used 'Type' parameter 
			["WebLang"]				= "0",			-- WEB language (0 - English)
				
		},
		["DATETIME"]		= {					-- display date format
			["TimeFormat"]	 		= "1",			-- 0-12 Hour, 1-24 Hour(default)
			["DateFormat"]	 		= "4",			-- 0:YYYY-MM-DD(default) 1:YYYY/MM/DD 2:DD-MM-YYYY 3:DD/MM/YYYY 4:WW-DD-MM 5:WW-MM-DD
			["Type"]                = "0",          -- 0:by date 1:by week
			["Offset"]              = "60",         -- offset, min -300~300
		},
		["DATEDISPLAY"]		= {
			["DisplayMode"] 		= "2",			-- 0:Disable 1:Day 2:Date 3:Rolling
		},
		["SNTP"]	  	 	= {					-- NTP Server settings
			["Enable"]				= "1",			-- 0:Disabled; 1:Enabled(default)
			["DTS"]                 = "2",          -- Daylight saving time mode. 0:Disbled; 1:Enabled; 2:Auto
			["NTPServer1"]			= "0.pool.ntp.org" -- Default phone value
		},
		["OTHERS"]			= {					-- Others settings
			["Telnet"]				= "0",			-- enable/disable Telnet service (by default enable)
			["QuickReset"]			= "1",			-- enale/disable long press OK/HOME to reset to factory settings
			["ViewRemoteHistory"]	= "1",			-- enable/disable using remote history in case missed calls
			["PowerLedDefault"]     = "0",          -- enable/disable power led
		},
		["SECURITY"]		= {					-- Security configuration
			["OnlyAcceptTrusted"]	= "0",			-- enable/disable accept untrusted certificates
		},
		["AUDIO"]           = {                 -- Phone -> Preference -> Key Press Sound
		    ["KeyVol"]              = "8",          -- value: 0-15
		    ["LossPktRateMin"]      = "100",          -- Minimum paket loss rate for screen notification. Value: 0-100, default 6.
		    ["LossPktRateMax"]      = "100",          -- Maximum paket loss rate for screen notification. Value: 0-100, default 12.
		},
		["DISPLAYSTYLE"]    = {                 -- Section to change display style 
		    ["PowerLedBlinkTimeOut"] = "0"         -- Power LED blinking timeout (1-14400 s), 0 - make blinking all time
		},
		["DNS"]    = {                          -- DNS settings
		    ["SipDnsCacheLevel"]    = "0",         -- Sip DNS cache
		},
	},

	["Network"]    	  = {					---  Network Configuation
	  	["LAN"]		   		= {					-- configure LAN port
	  		["Type"]				= nil,			-- 0:DHCP(default); 1:PPPoE; 2:Static IP;
	  	},	
		["PC"] 				= {					-- configure PC port 
			["Type"]				= "1",			-- 0:Router; 1:Bridge
		},
		["VLAN"]			= {					-- VLAN setting for LAN and PC port
		},
		["QOS"]				= {					-- QOS settings
			["SignalTos"]			= "46",			-- SIP QoS
			["RtpTos"]				= "46",			-- Voice QoS
		},
		["SNMP"]			= {					-- SNMP settings
			["Enable"]				= "0",			-- 0 - disable, 1 - enable
			["Port"]		 		= "",			-- SNMP port
			["TrustedAddress"]		= "",			-- SNMP access server
		},
		["LLDP"]			= {
			["LLDPEnable"]				= "0",
			["Interval"]			= "30",
		},
	},

	["Voice"]	  = {						--- Voice configuration
		["GENERAL"]	   		= {
			["VAD"]					= "0" -- WMS-10093
		},
		["JITTER"]	   		= {					-- Jitter Buffer
			["Enable"]				= "1",			-- enable/disable jitter buffer (enable by default)
			["Adaptive"]			= "0",			-- the type of jitter buffer: 0-Fixed, 1-Adaptive (default)
			["Min"]					= "0",			-- the minimum delay(0 by default)
			["Max"]					= "300",		-- maximum delay(300 by default)
			["Nominal"]				= "120",		-- normal delay (120 by default)
		},
		["TONE"]	   		= {					-- Jitter Buffer
			["Country"]				= "1"
		}
	},
	
	["Features"]  = {						--- Features configuration
		["REJECT"]	   		= {					-- return code for Reject
			["ReturnCode"]			= "603",		-- 404(Not Found); 480(Temporarily Unavailable); 486(Busy Here); 603(Decline)
		},
		["CALL_WAITING"] 	= {					-- call waiting feature settings
			["Enable"]				= "1",			-- enable/disable call waiting feature (enable by default)
			["PlayTone"]			= "1",			-- the phone to play warning tone when receiving an incoming call during an active call: 0:Disabled; 1:Enabled(default)
		},
		["DND"]				= {					-- DND feature settings
			["Enable"]				= "0",			-- enable/disable feature (disable by default)
		},
		["INTERCOM"]		= {					-- intercom feature settings
			["Enable"]				= "1",			-- enable/disable intercom (enable by default)
			["Mute"]				= "0",			-- enable/disable the phone to mute the speaker when automatically answer an intercom call (by default disable)
		},
		["SENDKEY"]			= {					-- send key settings
			["KeySendType"]			= "0",			-- 0 - disable, 1 - *, 2 - #
		},
		["ACD"]				= {					-- ACD Auto Available feature settings
			["ACDAutoAvailable"] 	= "0", 			-- enable/disable feature (disable by default)
		},
		["DIALNOW"]			= {					-- Dial Plan configuration
			["Delay"]				= "4",			-- inter digit timeout
			["DialAll"]				= "1",			-- enable/disbale to dialout timeout for all numbers (otherwise will be used only for numbers in dialrule list)
			["DialAllForHandfree"]	= "0",			-- enable/disbale to dialout timeout for all numbers in case using Handfree
			["DialAllForHeadset"]	= "0",			-- enable/disbale to dialout timeout for all numbers in case using Headset
			["OffHookDialOut"]      = "1",          -- enable/disable to dial now via handset pick up in phonebook and call history
		},
		["BLF"]				= {					-- BLF call type features
			["CallType"]			= "3",			-- 0: New call, 1: Blind Transfer, 3: The phone would put the number in dial interface and then transfer out automatically.
		},
		["BLFLED"]			= {					-- BLF LEDs configuration
			["TerminateForDual"]	= "0",			-- DSS key and Exp module LED (idle state): 0=OFF, 1=Green LED-ON, 2=Green LED slow flashing, 3=Green LED quick flashing. 4=Red LED-ON, 5=Red LED slow flashing, 6=Red LED quick flashing
			["ProceedingForDual"]	= "3",			-- DSS key and Exp module LED (ringing state)
			["ConfirmedForDual"]	= "1",			-- DSS key and Exp module LED (busy state)
			["ConfirmedHoldForDual"] = "1",			-- DSS key and Exp module LED (hold state)
			["TerminateForGreen"]	= "0",			-- Line key BLF LED (idle state): 0=OFF, 1=ON, 2=slow flashing, 3=quick flashing
			["ProceedingForGreen"]	= "3",			-- Line key BLF LED (ringing state)
			["ConfirmedForGreen"]	= "1"			-- Line key BLF LED (busy state)
		},
		["CALLTIMEOUT"]		= {					-- Call timeout settings
			["DialOut"]				= "3600",		-- timeout for outgoing calls
			["DialIn"]				= "3600",		-- timeout for incomfing calls
		},
		["Others"]       = {                    -- display/hide DTMF visualisation on display of the phone.
            ["ViewDTMFNum"]         = 1             --  0 - DTMF display=> *, 1 - DTMF dislpay=> specific digit
        },
	},
	
	["Programable"]    = {					--- Key/Display settings
		["LINEKEY1"]		= {					-- Line #1 settings
			["Type"]				= "14",			-- type of the line key (14 - Account)
			["Label"]				= "",			-- label for line key
			["Param1"]				= "",			-- not used for Account type (value field - BLF get notify with this extension)
			["Param2"]				= "1",			-- user account number		 (account field)
			["Param3"]				= "",			-- not used for Account type (extension field)
		},
		["LINEKEY2"]		= {					-- Line #2 settings
			["Type"]				= "14",			-- type of the line key (14 - Account)
			["Label"]				= "",			-- label for line key
			["Param1"]				= "",			-- not used for Account type
			["Param2"]				= "2",			-- user account number
			["Param3"]				= "",			-- not used for Account type
		},
		["LINEKEY3"]		= {					-- Line #3 settings
			["Type"]				= "14",			-- type of the line key (14 - Account)
			["Label"]				= "",			-- label for line key
			["Param1"]				= "",			-- not used for Account type
			["Param2"]				= "3",			-- user account number
			["Param3"]				= "",			-- not used for Account type
		},				
		["SOFTKEY01"]		= {					-- Soft Key #1 settings
			["Type"]				= "29",			-- type of the soft key (2 - Menu, 11 - History, 21 - XML browser, 25 - Simple Menu, 26 -VM, 27 - Transfer, 29 - XMLHistory, 30 - XMLPhonebook)
			["Label"]				= "History",	-- soft key label
			["Param1"]				= "",			-- value parameter 
			["Param2"]				= "",			-- account parameter
		},
		["SOFTKEY02"]		= {					-- Soft Key #2 settings
			["Type"]				= "30",			-- type of the soft key
			["Label"]				= "Phoneb",		-- soft key label
			["Param1"]				= "",			-- value parameter (xml browser url)
			["Param2"]				= "",			-- account parameter
		},
		["SOFTKEY03"]		= {					-- Soft Key #3 settings
			["Type"]				= "21",			-- type of the soft key
			["Label"]				= "Featur.",	-- soft key label
			["Param1"]				= "",			-- value parameter 
			["Param2"]				= "",			-- account parameter
		},
		["SOFTKEY04"]		= {					-- Soft Key #4 settings
			["Type"]				= "25",			-- type of the soft key
			["Label"]				= "Menu",		-- soft key label
			["Param1"]				= "",			-- value parameter 
			["Param2"]				= "",			-- account parameter
		},
		["OK"]				= {					-- 'OK' button settings
			["Type"]				= "4",			-- type of the button
			["Param1"]				= "",			-- not used for button settings
			["Param2"]				= "",			-- not used for button settings
		},
		["CANCEL"]				= {				-- 'CANCEL' button settings
			["Type"]				= "0",			-- type of the button
			["Param1"]				= "",			-- not used for button settings
			["Param2"]				= "",			-- not used for button settings
		},
		["BOOK"]				= {				-- 'BOOK' button settings
			["Type"]				= "30",			-- type of the button
			["Param1"]				= "",			-- xml broser url
			["Param2"]				= "",			-- not used for button settings
		},
		["MUTE"]				= {				-- 'MUTE' button settings
			["Type"]				= "0",			-- type of the button
			["Param1"]				= "",			-- not used for button settings
			["Param2"]				= "",			-- not used for button settings
		},
		["REDIAL"]				= {				-- 'REDIAL' button settings
			["Type"]				= "29",			-- type of the button
			["Param1"]				= "",			-- xml broser url
			["Param2"]				= "",			-- not used for button settings
		},
		["VOICEMESSAGE"]		= {				-- 'VOICEMESSAGE' button settings
			["Type"]				= "26",			-- type of the button
			["Param1"]				= "",			-- not used for button settings
			["Param2"]				= "",			-- not used for button settings
		},
		["TRANSFER"]            = {              -- 'TRANSFER' button settings
			["Type"]				= "27",			-- type of the button
			["Param1"]				= "",			-- not used for button settings
			["Param2"]				= "",			-- not used for button settings
		}
	},	

	["Account1"]  	  = {			   		--- Settings for account #1 (settings are filled in dynamic section)
		["GENERAL"] 		= {					-- general settings
			["Enable"]				= "1",			-- enable/disable account
		},
		["SIP"]				= {					-- SIP settings
			["Server2"]				= "",			-- IP address for second SIP server
			["TransType"]			= "0",			-- the transport type: 0:UDP(Default); 1:TCP; 2:TLS;
			["ListenPortMin"] 		= "1024", 		-- miminum listen port
			["ListenPortMax"] 		= "65535", 		-- maximum listen port
		},			
		["OUTPROXY"]		= {					-- outbound proxy server settings
			["Enable"]				= "0",			-- enable/disable outbound proxy
			["Server"]              = "",           -- main Server IP of the outbound proxy
			["Port"]                = "",           -- main Server port of the outbound proxy
			["BakServer"]			= "",			-- backup Server IP of the outbound proxy
			["BakPort"]             = "",           -- backup Server port of the outbound proxy
		},			
		["STUN"]			= {					-- STUN settings
			["Enable"]				= "0",			-- enable/disable STUN feature
		},
		["ENCRYPTION"] 		= {					-- SRTP encryption settings
			["SRTPEncryption"]    	= "2",	 		-- 0:Disabled, 1:Enabled, 2:Optional (disabled by default)
			["SRTPEncryptionAlg"] 	= "",	 		-- 0:AES_CM_128_HMAC_SHA1_80, 1:AES_CM_128_HMAC_SHA1_32, 2:F8_128_HMAC_SHA1_80 (all by default)
		},
		["DTMF"]			= {					-- DTMF settings
			["Type"]				= "3",			-- 1:Inband; 2:RFC2833(Default); 3:Info; 4:Inband + Info; 5:Info + RFC2833;
			["InfoType"]			= "2",			-- 0:Disabled(default); 1:DTMF; 2:DTMF-Relay; 3:Telephone-Event;
			["Payload"]				= "101",		-- the RFC2833 payload, ranges from 96 to 225 (101 by default); 
			["Duration"]			= "120",		-- DTMF tone len (default 300 sec)
		},
		["NAT"]				= {					-- NAT traversal settings
			["NatTraversal"] 		= "0",			-- disable/enable NAT traversal
		},
		["BLF"]				= {					-- BLF settings
			["SubscribePeriod"] 	= "1200", 		-- BLF subscribe period (1800 seconds by default)
			["BlfNotifyParseType"]  = "1",          -- BLF will blinks mode (1 - active + incoming call, 0 - active call)
		},
		["VOICEMSG"]		= {					-- the voice message number settings
			["Number"]				= "",
		},
		["MUSIC_SERVER"] 	= {					-- Music Server settings
			["Enable"]				= "0",			-- enable/disable Music Server
		},
		["ANONYMOUS_CALL"] 	= {					-- AnonymousCall configuration
			["Enable"]				= "0",			-- enable/disable anonymous call
		},
		["REJECT_ANONYMOUSCALL"] = { 			-- Reject AnonymousCall configuration
			["Enable"]				= "0",			-- enable/disable reject anonymous call
		},
		["REG"]				= {					-- Register expire time settings
			["Timeout"]				= "300",		-- registration period for SIP Server 1
			["Timeout2"]			= "300",		-- registration period for SIP Server 2
		},
		["AUTO_ANSWER"]		= {					-- settings for auto answer when receiving a incoming call
			["Enable"]				= "0",			-- enable/disable feature
		},
		["SESSION"]			= {					-- Settion timer settings
			["EnableTimer"]			= "0",			-- enable/disable session timer
			["Interval"]			= "7200",		-- session timer interval (seconds)
			["Refresher"]			= "0",			-- Session Refresher (0 - UAC, 1 - UAS)	
		},
		["SUBSCRIBE"]		= {					-- Subscribe settings
			["SubsribeMWI"]		   	= "1",			-- subscribe the MWI status; 0:Disabled(Default); 1:Enabled
			["SubscribeMWIExpire"] 	= "3600",		-- the subscribe period of the MWI(in seconds), the range is 1800~65535, default is 1800;
			["SubscribeFKSSExpire"]	= "1800",		-- ACD Expire
		},
		["CALL"]			= {					-- Call settings
			["Enable100Rel"] 		= "0",			-- 100 reliable retransmission: 0:Disabled(Default); 1:Enabled
			["CIDSource"]			= "1",			-- caller id source: 1 - PAI
			["Ptime"]				= "20",			-- the RTP packet time; 0(Disabled), 10, 20, 30, 40, 50, 60;
			["UserPhone"]			= "0",			-- enable/disable using user=phone
			["UseRemoteCodec"]		= "0",			-- set local/remote codec prioiry (wideband - remote codec priority, narrowband - local codec priority)
		},
		["FEATURE_DND"]		= {					-- DND feature settings
			["Enable"]				= "0",
		},
		["FEATURE_FORWARD_ALWAYS"] = {			-- FCF feature settings
			["Enable"]				= "0",
		},
		["FEATURE_FORWARD_BUSY"] = {			-- BCF feature settings
			["Enable"]				= "0",
		},
		["FEATURE_FORWARD_TIMEOUT"] = {			-- UCF feature settings
			["Enable"]				= "0",		
		},
		["ADVANCED"]		= {					-- Andvansed settings
			["ShowMissedCall"] 		= "1", 			-- enable/disable Missed Call Log
		},
		["ESCAPE_CHARACTER"] = {				-- escaping non Ascii character
			["Enable"]				= "1",
		},
	},
	
	["Forward"]		  = {					--- Call Forward Configuration
		["FORWARDMODE"]		= {					-- Call feature mode settings
			["FeatureSync"]			= "1",			-- enable/disable Feature Key Sync (send SUBSCRIBE as-feature-event)
			["Mode"] 				= "1",			-- forward mode (0: phone, 1: custom)
		}
	},
	
	["XMLBrowser"]	  = {					--- XML browser settings
	 	["REMOTEURL"] 		= {					--- Url for XML phonebook and XML history
	 	 	["History"]				= "",			-- Url for XML history
	 	 	["Phonebook"]			= "",			-- Url for XML phonebook
            ["LiveSearch"]          = "",           -- Url for live search feature
	 	},
	},
	
	["CustomMenu"]    = {					--- XXX: should be removed in next version. Dial Menu settings for Select button
		["DIALMENUITEM01"]      = {                                     -- item for the Dial Menu
			["Type"]                                = "7",                  -- type of the item (1 - LocalPB, 2 - RemotePB, 3 - LDAP, 4 - BroadsoftPB, 5 - XMLHistory, 6 - XMLPhonebook, 7 - XMLBrowser)
            ["Label"]                               = "",                   -- label for menu
			["Param1"]                              = "",                   -- url in case XML browser
		},
		["DIALMENUITEM02"]      = {
			["Type"]                                = "7",
			["Label"]                               = "",
			["Param1"]                              = "",
		},
		["DIALMENUITEM03"]      = {
			["Type"]                                = "",
			["Label"]                               = "",
			["Param1"]                              = "",
		},
		["DIALMENUITEM04"]      = {
			["Type"]                                = "",
			["Label"]                               = "",
			["Param1"]                              = "",
		},
		["DIALMENUITEM05"]      = {
			["Type"]                                = "",
			["Label"]                               = "",
			["Param1"]                              = "",
		},
		["DIALMENUITEM06"]      = {
			["Type"]                                = "",
			["Label"]                               = "",
			["Param1"]                              = "",
		},
		["DIALMENUITEM07"]      = {
			["Type"]                                = "",
			["Label"]                               = "",
			["Param1"]                              = "",
		},
		["DIALMENUITEM08"]      = {
			["Type"]                                = "",
			["Label"]                               = "",
			["Param1"]                              = "",
		},
	},
		
	["Multicast"]	  = {					--- Multicast paging service
		["PRIORITY"]		= {					-- Multicast priority settings
			["Enable"]				= "0",			-- Paging Priority Active
			["Priority"]			= "0",			-- Paging Barge
		},
		["GENERAL"]			= {					-- Multicast general settings
			["Codec"]				= "8"			-- Multicase codec (0 - PCMU, 8 - PCMA, 9 - G722, 18 - G729)
		},
		["LISTEN0"]			= {					-- Configuration for multicast service
			["Label"]				= "",			-- Label of the multicast service
			["IP"]					= "",			-- IP of the mutlicase service
		},
		["LISTEN1"]			= {					
			["Label"]				= "",			
			["IP"]					= "",			
		},
		["LISTEN2"]			= {					
			["Label"]				= "",			
			["IP"]					= "",			
		},
		["LISTEN3"]			= {					
			["Label"]				= "",			
			["IP"]					= "",			
		},
		["LISTEN4"]			= {					
			["Label"]				= "",			
			["IP"]					= "",			
		},
		["LISTEN5"]			= {					
			["Label"]				= "",			
			["IP"]					= "",			
		},
		["LISTEN6"]			= {					
			["Label"]				= "",			
			["IP"]					= "",			
		},
		["LISTEN7"]			= {					
			["Label"]				= "",			
			["IP"]					= "",			
		},
		["LISTEN8"]			= {					
			["Label"]				= "",			
			["IP"]					= "",			
		},
		["LISTEN9"]			= {					
			["Label"]				= "",			
			["IP"]					= "",			
		}
	},
	
	["Tone"]  = {
		["GENERAL"]  = {
			["Busy"]       = "",
			["Ringback"]   = "",
			["Congestion"] = "",
			["Dialtone"]   = "",
			["Callwait"]   = ""
		}
	}
}
